package cz.ceskahospodynka.mobile.core.database.hospodynka;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.channels.FileChannel;

import cz.ceskahospodynka.mobile.core.database.DatabaseInfo;
import cz.ceskahospodynka.mobile.core.database.core.Generator;
import cz.ceskahospodynka.mobile.core.database.hospodynka.dao.CategoryModel;
import cz.ceskahospodynka.mobile.core.database.hospodynka.dao.PointDAO;
import cz.ceskahospodynka.mobile.core.database.hospodynka.dao.RecipeModel;
import cz.ceskahospodynka.mobile.core.database.hospodynka.handlers.CategoryHandler;

/**
 * Created with IntelliJ IDEA.
 * User: tomas
 * Date: 14.11.13
 * Time: 23:20
 * To change this template use File | Settings | File Templates.
 */
public class Creator extends SQLiteOpenHelper {

    public Creator(Context context) {
        super(context, DatabaseInfo.DATABASE_NAME, null, DatabaseInfo.DATABASE_VERSION);
        SQLiteDatabase db = getWritableDatabase();
        db.close();

        boolean isDatabase = isDatabase();
        Log.d("TEST", "is database: " + isDatabase);
        if(!isDatabase){
            importDatabase(context);
        }
    }

    private boolean isDatabase(){
        Log.d("TEST", "copy database start");
        CategoryHandler categoryHandler = new CategoryHandler();
        SQLiteDatabase db = getReadableConnection();
        try {
            return categoryHandler.getCount(db) > 0;
        }finally {
            db.close();
        }
    }

    private boolean importDatabase(Context context) {
        close();

        try {
            InputStream source = context.getAssets().open(DatabaseInfo.DATABASE_NAME + ".db");
            File oldDb = context.getDatabasePath(DatabaseInfo.DATABASE_NAME);
            fileCopy(source, oldDb);
            getWritableDatabase().close();
            Log.d("TEST", "copy database stop");
            return true;
        } catch (IOException e){
            Log.d("TEST", e.getMessage(), e);

            return false;
        }

    }

    public void fileCopy(InputStream in, File dst) throws IOException {
        OutputStream out = new FileOutputStream(dst);

        byte[] buf = new byte[1024];
        int len;
        while ((len = in.read(buf)) > 0) {
            out.write(buf, 0, len);
        }
        in.close();
        out.close();
    }

    @Override
    public void onCreate(SQLiteDatabase db) {
        CategoryModel categoryModel = new CategoryModel();
        Generator gen = new Generator(categoryModel);
        Log.d("database creator", gen.createTable());
        db.execSQL(gen.createTable());

        RecipeModel recipeModel = new RecipeModel();
        gen = new Generator(recipeModel);
        Log.d("database creator", gen.createTable());
        db.execSQL(gen.createTable());

        PointDAO pointDAO = new PointDAO();
        gen = new Generator(pointDAO);
        Log.d("database creator", gen.createTable());
        db.execSQL(gen.createTable());
    }

    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        CategoryModel categoryModel = new CategoryModel();
        Generator gen = new Generator(categoryModel);
        Log.d("database creator", gen.dropTable());
        db.execSQL(gen.dropTable());

        RecipeModel recipeModel = new RecipeModel();
        gen = new Generator(recipeModel);
        Log.d("database creator", gen.dropTable());
        db.execSQL(gen.dropTable());

        PointDAO pointDAO = new PointDAO();
        gen = new Generator(pointDAO);
        Log.d("database creator", gen.dropTable());
        db.execSQL(gen.dropTable());

        onCreate(db);
    }

    public SQLiteDatabase getWritableConnection() {
        return this.getWritableDatabase();
    }

    public SQLiteDatabase getReadableConnection() {
        return this.getReadableDatabase();
    }
}
