package cz.ceskahospodynka.mobile.core.database.hospodynka;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.util.Log;

import com.google.gson.Gson;

import cz.ceskahospodynka.mobile.core.Settings;
import cz.ceskahospodynka.mobile.core.beans.Recipe;
import cz.ceskahospodynka.mobile.core.database.hospodynka.handlers.RecipeHandler;
import cz.ceskahospodynka.mobile.core.utils.AES;
import cz.ceskahospodynka.mobile.core.utils.IO;
import cz.ceskahospodynka.mobile.ui.BaseActivity;
import cz.ceskahospodynka.mobile.ui.adapters.RecipeItem;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

/**
 * Created by tomas on 21.4.14.
 */
public class RecipeLoader {

    private static final String TAG = "RECIPE_LOADER";

    private Context context;
    private RecipeHandler recipeHandler = new RecipeHandler();

    public RecipeLoader(Context context) {
        this.context = context;
    }

    public List<RecipeItem> loadRecipesFromURL(String url) {
        return loadRecipesFromURL(url, false);
    }

    public List<RecipeItem> loadRecipesFromURL(String url, boolean firstEmpty) {
        return loadRecipesFromURL(url, Settings.IMAGE_SMALL, firstEmpty);
    }

    public List<RecipeItem> loadRecipesFromURL(String url, String imageSize) {
        return loadRecipesFromURL(url, imageSize, false);
    }

    public List<RecipeItem> loadRecipesFromURL(String url, String imageSize, boolean firstEmpty) {
        List<RecipeItem> recipes = new LinkedList<RecipeItem>();
        if(firstEmpty){
            recipes.add(new RecipeItem());
        }

        if(context instanceof BaseActivity && ((BaseActivity)context).isOnline()) {

            try {

                String json = IO.readUrl(url);
                Gson gson = new Gson();
                Recipe recipe = gson.fromJson(json, Recipe.class);

                if(context == null){
                    return recipes;
                }

                if(context == null){
                    return recipes;
                }


                if(context == null){
                    return recipes;
                }
                Creator creator = new Creator(context);
                SQLiteDatabase db = creator.getWritableConnection();
                try {
                    RecipeHandler recipeHandler = new RecipeHandler();
                    for (Recipe.Item i : recipe.getRecipeList()) {

                        boolean favorite = recipeHandler.isRecipe(db, i.getId());
                        RecipeItem item = new RecipeItem(
                                i.getId(),
                                AES.decrypt(i.getNazev()),
                                AES.decrypt(i.getHlavs()),
                                i.getBody(),
                                String.format(imageSize, i.getId()),
                                null,
                                favorite,
                                AES.decrypt(i.getSuroviny()),
                                AES.decrypt(i.getPostup()),
                                AES.decrypt(i.getAutor()),
                                AES.decrypt(i.getPod_kategorie())
                        );
                        recipes.add(item);
                    }
                } finally {
                    db.close();
                }
            } catch (IOException e) {
                Log.e(TAG, e.getMessage(), e);
            }
        }
        return recipes;
    }

    public static RecipeItem decryptRecipe(Recipe.Item i, boolean favorite, String imageType){
        return new RecipeItem(
                i.getId(),
                AES.decrypt(i.getNazev()),
                AES.decrypt(i.getHlavs()),
                i.getBody(),
                String.format(Settings.API_RECIPES_IMAGE, Settings.API_VERSION, imageType, i.getId()),
                null,
                favorite,
                AES.decrypt(i.getSuroviny()),
                AES.decrypt(i.getPostup()),
                AES.decrypt(i.getAutor()),
                AES.decrypt(i.getPod_kategorie())
        );
    }

    public static RecipeItem decryptRecipe(Recipe.Item i, boolean favorite){
            return decryptRecipe(i, favorite, Settings.IMAGE_MEDIUM);
    }
}
