package cz.ceskahospodynka.mobile.core.database.hospodynka.handlers;

import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.graphics.BitmapFactory;

import cz.ceskahospodynka.mobile.core.database.core.RS;
import cz.ceskahospodynka.mobile.core.database.hospodynka.dao.RecipeModel;

/**
 * Created with IntelliJ IDEA.
 * User: tomas
 * Date: 13.11.13
 * Time: 17:18
 * To change this template use File | Settings | File Templates.
 */
public class RecipeHandler extends DefaultHandler {

    public static final byte FAVORITE_SIZE = 8;

    public RecipeHandler() {
        super(new RecipeModel());
    }

    /**
     * @param db - readeble database
     */
    public boolean isRecipe(SQLiteDatabase db, Integer id) {
        RecipeModel recipeModel = (RecipeModel) model;

        Cursor cursor = db.query(recipeModel.getTableName(), new String[]{
                        recipeModel.getRecipeIdColumn(),
                }, recipeModel.getRecipeIdColumn() + "=?",
                new String[]{String.valueOf(id)}, null, null, null, null
        );
        try {
            if (cursor != null) {
                cursor.moveToFirst();
                if (cursor.getCount() > 0) {
                    return true;
                }
            }
            return false;
        } finally {
            cursor.close();
        }
    }

    /**
     * @param db - readeble database
     */
    public boolean dropRecipe(SQLiteDatabase db, Integer id) {
        RecipeModel recipeModel = (RecipeModel) model;
        return db.delete(recipeModel.getTableName(), recipeModel.getRecipeIdColumn() + "=" + id, null) > 0;
    }

    /**
     * @param db - readeble database
     */
    public RecipeModel findRecipe(SQLiteDatabase db, Integer id) {
        RecipeModel recipeModel = (RecipeModel) model;

        Cursor cursor = db.query(recipeModel.getTableName(), new String[]{
                        recipeModel.getRecipeNameColumn(),
                        recipeModel.getRecipeImageColumn(),
                        recipeModel.getRecipeIngredientColumn(),
                        recipeModel.getRecipeAdvanceColumn(),
                }, recipeModel.getRecipeIdColumn() + "=?",
                new String[]{String.valueOf(id)}, null, null, null, null
        );
        try {
            if (cursor != null) {
                cursor.moveToFirst();

                if (cursor.getCount() == 0) {
                    return null;
                }

                RecipeModel recipe = new RecipeModel();
                recipe.recipeName = cursor.getString(cursor.getColumnIndex(recipeModel.getRecipeNameColumn()));
                recipe.recipeIngredient = cursor.getString(cursor.getColumnIndex(recipeModel.getRecipeIngredientColumn()));
                recipe.recipeAdvance = cursor.getString(cursor.getColumnIndex(recipeModel.getRecipeAdvanceColumn()));
                recipe.recipeName = cursor.getString(cursor.getColumnIndex(recipeModel.getRecipeNameColumn()));

                byte[] bb = cursor.getBlob(cursor.getColumnIndex(recipe.getRecipeImageColumn()));
                if (bb != null) {
                    recipe.recipeImage = BitmapFactory.decodeByteArray(bb, 0, bb.length);
                }

                return recipe;
            }
            return null;
        } finally {
            cursor.close();
        }
    }

    public void fetch(SQLiteDatabase db, RS rs) {
        fetch(db, rs, false);
    }

    public void fetch(SQLiteDatabase db, RS rs, boolean allData) {

        RecipeModel recipeModel = (RecipeModel) model;

        Cursor cursor = db.query(recipeModel.getTableName(), new String[]{
                recipeModel.getRecipeIdColumn(),
                recipeModel.getRecipeNameColumn(),
                recipeModel.getRecipeImageUrlColumn(),
                recipeModel.getRecipeMainIngredientColumn(),
                recipeModel.getRecipePointsColumn(),
                recipeModel.getRecipeImageColumn()
        }, null, null, null, null, null, (allData ? null : FAVORITE_SIZE + ""));
        try {
            if (cursor.moveToFirst()) {
                do {
                    RecipeModel model = new RecipeModel();
                    model.recipeId = cursor.getInt(cursor.getColumnIndex(model.getRecipeIdColumn()));
                    model.recipeName = cursor.getString(cursor.getColumnIndex(model.getRecipeNameColumn()));
                    model.recipeImageUrl = cursor.getString(cursor.getColumnIndex(model.getRecipeImageUrlColumn()));
                    model.recipeMainIngredient = cursor.getString(cursor.getColumnIndex(model.getRecipeMainIngredientColumn()));
                    model.recipePoints = cursor.getInt(cursor.getColumnIndex(model.getRecipePointsColumn()));
                    byte[] imgByte = cursor.getBlob(cursor.getColumnIndex(model.getRecipeImageColumn()));

                    if(imgByte != null){
                        model.recipeImage = BitmapFactory.decodeByteArray(imgByte, 0, imgByte.length);
                    }

                    rs.fetch(model);
                } while (cursor.moveToNext());
            }
        } finally {
            cursor.close();
        }
    }
}
