package cz.ceskahospodynka.mobile.core.tasks;

import android.os.AsyncTask;
import android.util.Log;

import org.apache.http.message.BasicNameValuePair;
import cz.ceskahospodynka.mobile.core.Settings;
import cz.ceskahospodynka.mobile.core.utils.ServerCommunication;

/**
 * Created by tomas, Medoro s.r.o. on 18.4.15.
 */
public abstract class AndroidTask {

    private static final String TAG = "Android task";

    protected ServerCommunication serverCommunication;

    public AndroidTask() {
            serverCommunication = new ServerCommunication();
    }

    public <T> T task(final Class<T> outputType, final AfterExecutor afterExecutor) {

        new AsyncTask<Void, T, T>() {

            @Override
            protected void onPreExecute() {
                super.onPreExecute();
                beforeTask();
            }

            @Override
            protected T doInBackground(Void... voids) {
                try {
                    serverCommunication.getParams().add(new BasicNameValuePair(Settings.PARAM_API, Settings.API_VERSION));
                    return perform(outputType);
                } catch (Exception e) {
                    Log.w(TAG, e);
                }
                return null;
            }

            @Override
            protected void onPostExecute(T t) {
                super.onPostExecute(t);
                if (afterExecutor != null) {
                    afterExecutor.work(t);
                }
            }
        }.execute();

        return null;
    }

    public <T> T task(final Class<T> outputType) throws Exception {
        return task(outputType, null);
    }

    public void task() throws Exception {
        task(Void.class);
    }

    public void task(final AfterExecutor afterExecutor) throws Exception {
        task(Void.class, afterExecutor);
    }

    abstract protected <T> T perform(Class<T> outputType) throws Exception;

    public void beforeTask(){

    }
}
