package cz.ceskahospodynka.mobile.core.utils;

import android.util.Log;

import com.google.gson.Gson;
import com.google.gson.JsonParseException;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.apache.http.protocol.HTTP;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by tomas, Medoro s.r.o. on 18.4.15.
 */
public class ServerCommunication {

    private Gson gson = new Gson();

    private List<NameValuePair> params = new ArrayList<NameValuePair>();
    private String response = null;

    public <T> T getResponse(String address, Class<T> type) throws Exception {

        String response = readUrl(address);

        if(response.length() == 0){
            throw new RuntimeException("Server response return null");
        }

        try{
            return gson.fromJson(response, type);
        } catch (JsonParseException e){
            throw new RuntimeException("JsonParseException: " + e.getMessage());
        }
    }

    /**
     * Provede post
     * @throws java.io.IOException post selhal
     */
    public void post(String url) {

        Log.d("URL: ", "url: " + url);

        try {

            HttpParams httpParameters = new BasicHttpParams();
            int timeoutConnection = 1000 * 60 * 3;
            HttpConnectionParams.setConnectionTimeout(httpParameters, timeoutConnection);
            int timeoutSocket = 1000 * 60 * 3;
            HttpConnectionParams.setSoTimeout(httpParameters, timeoutSocket);

            HttpClient httpclient = new DefaultHttpClient(httpParameters);
            HttpPost httppost = new HttpPost(url);
            httppost.setEntity(new UrlEncodedFormEntity(params, HTTP.UTF_8));

            HttpResponse httpResponse = httpclient.execute(httppost);

            if(httpResponse.getEntity() == null) {

                HttpClient subClient = new DefaultHttpClient();
                HttpPost subPost = new HttpPost(url);

                for(NameValuePair par : params){
                    Log.d("TEST", par.getName() + ": " + par.getValue());
                }

                subPost.setEntity(new UrlEncodedFormEntity(params, HTTP.UTF_8));

                httpResponse = subClient.execute(subPost);

            }

            InputStream ips = httpResponse.getEntity().getContent();
            BufferedReader reader = new BufferedReader(new InputStreamReader(ips, "UTF-8"));

            String str;
            StringBuffer respBuf = new StringBuffer();
            while ((str = reader.readLine()) != null) {
                respBuf.append(str);
                respBuf.append('\n');
            }
            reader.close();

            if (respBuf.length() > 0) {
                response = respBuf.toString();
            }
        } catch (ClientProtocolException e){
            Log.w(e.getMessage(), e);
        } catch (IOException e){
            Log.w(e.getMessage(), e);
        }
    }

    private String readUrl(String urlString) throws IOException {
        BufferedReader reader = null;
        try {
            URL url = new URL(urlString);

            HttpURLConnection huc = (HttpURLConnection) url.openConnection();
            HttpURLConnection.setFollowRedirects(false);
            huc.setConnectTimeout(15 * 1000);
            huc.setRequestMethod("GET");
            huc.setRequestProperty("User-Agent", "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.2) Gecko/20090729 Firefox/3.5.2 (.NET CLR 3.5.30729)");
            huc.connect();

            reader = new BufferedReader(new InputStreamReader(huc.getInputStream()));
            StringBuffer buffer = new StringBuffer();
            int read;
            char[] chars = new char[1024];
            while ((read = reader.read(chars)) != -1)
                buffer.append(chars, 0, read);

            return buffer.toString();
        } finally {
            if (reader != null)
                reader.close();
        }
    }

    /**
     * Vrati odpoved, kterou vratil server
     * @return odpoved serveru
     */
    public String getResponse() {
        return response;
    }

    public List<NameValuePair> getParams() {
        return params;
    }

    public Gson getGson() {
        return gson;
    }

}
