package cz.ceskahospodynka.mobile.ui;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.database.sqlite.SQLiteDatabase;
import android.graphics.Shader;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.support.v4.view.MenuItemCompat;
import android.support.v7.widget.SearchView;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.GridView;
import android.widget.Toast;

import com.google.android.gms.analytics.HitBuilders;
import com.google.android.gms.analytics.Tracker;
import com.google.gson.Gson;

import org.uhk.ceskahospodynka.mobile.R;

import java.util.LinkedList;
import java.util.List;

import cz.ceskahospodynka.mobile.application.HospodynkaApplication;

import cz.ceskahospodynka.mobile.core.database.core.RS;
import cz.ceskahospodynka.mobile.core.database.core.TableModel;
import cz.ceskahospodynka.mobile.core.database.hospodynka.Creator;
import cz.ceskahospodynka.mobile.core.database.hospodynka.dao.CategoryModel;
import cz.ceskahospodynka.mobile.core.database.hospodynka.handlers.CategoryHandler;
import cz.ceskahospodynka.mobile.core.utils.CategoryStack;
import cz.ceskahospodynka.mobile.ui.adapters.CategoryAdapter;
import cz.ceskahospodynka.mobile.ui.adapters.CategoryGridItem;
import roboguice.activity.RoboActionBarActivity;

/**
 * Created by tomas, Medoro s.r.o. on 26.11.14.
 */
public class BaseActivity extends RoboActionBarActivity implements SearchView.OnQueryTextListener {

    private SearchView searchView;
    protected Menu optionsMenu;

    protected Gson gson = new Gson();

    int PRIVATE_MODE = 0;
    private static final String PREFERENCE_ID = "ceskahospodynka";

    public static final String PREF_FACEBOOK_USER_UID = "facebook_id";
    public static final String PREF_FACEBOOK_USER_NAME = "facebook_name";
    public static final String PREF_GOOGLE_USER_UID = "google_id";
    public static final String PREF_GOOGLE_USER_NAME = "google_name";
    public static final String PREF_CH_USERNAME = "ch_name";
    public static final String PREF_CH_PASSWORD = "ch_password";

    private SharedPreferences sharedPreferences;
    private SharedPreferences.Editor editor;

    private boolean showNotConnectionInfo = true;

    protected GridView categoryGrid;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // nastaveni stylu baru
        BitmapDrawable bg = (BitmapDrawable) getResources().getDrawable(R.drawable.bg_striped);
        bg.setTileModeXY(Shader.TileMode.REPEAT, Shader.TileMode.REPEAT);
        getSupportActionBar().setBackgroundDrawable(bg);

        BitmapDrawable bgSplit = (BitmapDrawable) getResources().getDrawable(R.drawable.bg_striped_split_img);
        bgSplit.setTileModeXY(Shader.TileMode.REPEAT, Shader.TileMode.REPEAT);
        getSupportActionBar().setSplitBackgroundDrawable(bgSplit);

        // nastaveni klikatelne ikony home
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        View someView = findViewById(R.id.activity);
        View root = someView.getRootView();
        root.setBackgroundColor(getResources().getColor(R.color.background));

        if(showNotConnectionInfo && !isOnline()){
            showNotConnectionInfo();
        }

        sharedPreferences = getSharedPreferences(PREFERENCE_ID, PRIVATE_MODE);
        editor = sharedPreferences.edit();
    }

    protected void initCategories(boolean initAdapter) {

        categoryGrid = (GridView) findViewById(R.id.category_grid);

        if (initAdapter){
            Creator creator = new Creator(this);
            SQLiteDatabase db = creator.getWritableConnection();

            try {
                final List<CategoryGridItem> categories = new LinkedList<CategoryGridItem>();

                CategoryHandler categoryHandler = new CategoryHandler();
                categoryHandler.fetch(db, new RS() {
                    @Override
                    public void fetch(TableModel model) {
                        categories.add(new CategoryGridItem(((CategoryModel) model).categoryName, ((CategoryModel) model).categoryImage));
                    }
                });

                categoryGrid.setAdapter(new CategoryAdapter(this, categories));
                categoryGrid.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                    @Override
                    public void onItemClick(AdapterView<?> parent, View v, int position, long id) {

                        Intent intent = new Intent(BaseActivity.this, RecipesActivity.class);
                        Bundle bundle = new Bundle();

                        CategoryStack categoryStack = new CategoryStack(categories.get(position).getTitle());
                        bundle.putString(RecipesCategoriesActivity.EXTRA_CATEGORY_INFO, gson.toJson(categoryStack));

                        intent.putExtras(bundle);
                        startActivity(intent);
                    }
                });
            } finally {
                db.close();
            }
        }
    }

    @Override
    protected void onResume() {
        super.onResume();

        Tracker t = ((HospodynkaApplication) getApplication()).getTracker(HospodynkaApplication.HospodynkaTracker.APP_TRACKER);
        t.send(new HitBuilders.EventBuilder()
                .setCategory("ActivitiesMovement")
                .setAction(this.getClass().getSimpleName())
                .setLabel("onResume")
                .setValue(1)
                .build());
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        this.optionsMenu = menu;
        getMenuInflater().inflate(R.menu.main_menu, menu);

        MenuItem searchItem = menu.findItem(R.id.ab_search);
        searchView = (SearchView) MenuItemCompat.getActionView(searchItem);
        searchView.setOnQueryTextListener(this);

        return super.onCreateOptionsMenu(menu);
    }

    public boolean onOptionsItemSelected(MenuItem item) {

        Intent myIntent;
        switch (item.getItemId()) {
            case android.R.id.home:
                onBackPressed();
                return true;
            case R.id.root_menu:
                return false;
            case R.id.action_main:
                myIntent = new Intent(this, HospodynkaActivity.class);
                startActivity(myIntent);
                return true;
            case R.id.action_my_favorite:
                myIntent = new Intent(this, MyRecipesActivity.class);
                startActivity(myIntent);
                return true;
            case R.id.action_recipes:
                myIntent = new Intent(this, RecipesCategoriesActivity.class);
                startActivity(myIntent);
                return true;
            case R.id.action_news:
                myIntent = new Intent(this, NewsActivity.class);
                startActivity(myIntent);
                return true;
            case R.id.action_opinions:
                myIntent = new Intent(this, LastOpinionsActivity.class);
                startActivity(myIntent);
                return true;
            case R.id.action_favorite:
                myIntent = new Intent(this, BestRecipesActivity.class);
                startActivity(myIntent);
                return true;
            case R.id.action_menu:
                myIntent = new Intent(this, MenusActivity.class);
                startActivity(myIntent);
                return true;
            case R.id.action_link:
                myIntent = new Intent(this, LinkActivity.class);
                startActivity(myIntent);
                return true;
            case R.id.action_articles:
                myIntent = new Intent(this, ArticlesActivity.class);
                startActivity(myIntent);
                return true;
            case R.id.action_search:
                myIntent = new Intent(this, SearchRecipesActivity.class);
                startActivity(myIntent);
                return true;
            case R.id.action_help:
                myIntent = new Intent(this, HelpActivity.class);
                startActivity(myIntent);
                return true;
            case R.id.ab_search:
                searchView.setIconified(false);
                return true;
            default:
                Toast.makeText(getApplicationContext(), "Nothing here!", Toast.LENGTH_LONG).show();
                return false;

        }
    }

    public boolean isOnline() {
        ConnectivityManager cm = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo netInfo = cm.getActiveNetworkInfo();
        return netInfo != null && netInfo.isConnectedOrConnecting();
    }

    private void showNotConnectionInfo(){
        final AlertDialog alertDialog = new AlertDialog.Builder(this).create();
        alertDialog.setTitle(R.string.not_internet_title);
        alertDialog.setMessage(getResources().getString(R.string.not_internet));
        alertDialog.setIcon(R.drawable.ic_launcher_bw_48);
        alertDialog.setButton("OK", new DialogInterface.OnClickListener(){
            public void onClick(DialogInterface dialog, int which) {
                alertDialog.dismiss();
            }
        });
        alertDialog.show();
    }

    public void addString(String key, String value){
        editor.putString(key, value);
        editor.commit();
    }

    public String getString(String key){
        return sharedPreferences.getString(key, null);
    }

    public void setShowNotConnectionInfo(boolean showNotConnectionInfo) {
        this.showNotConnectionInfo = showNotConnectionInfo;
    }

    @Override
    public boolean onQueryTextSubmit(String searchString) {
        Intent intent = new Intent(this, SearchRecipesActivity.class);
        Bundle searchBundle = new Bundle();
        searchBundle.putString(SearchRecipesActivity.BUNDLE_SEARCH_TEXT, searchString);
        intent.putExtras(searchBundle);
        startActivity(intent);
        return true;
    }

    @Override
    public boolean onQueryTextChange(String s) {
        return false;
    }

    public enum Device{
        PHONE, TABLET_7, TABLET_10
    }

    protected Device detectDevice(){
        DisplayMetrics metrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(metrics);

        int widthPixels = metrics.widthPixels;
        int heightPixels = metrics.heightPixels;

        float scaleFactor = metrics.density;

        float widthDp = widthPixels / scaleFactor;
        float heightDp = heightPixels / scaleFactor;

        float smallestWidth = Math.min(widthDp, heightDp);

        Log.d("TEST", "width: " + smallestWidth);

        if (smallestWidth > 720) {
            //Device is a 10" tablet
            return Device.TABLET_10;
        }
        else if (smallestWidth > 600) {
            //Device is a 7" tablet
            return Device.TABLET_7;
        }
        return Device.PHONE;
    }
}
