package cz.ceskahospodynka.mobile.ui;

import android.content.Intent;
import android.os.Bundle;
import android.text.Html;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.squareup.picasso.Picasso;

import org.uhk.ceskahospodynka.mobile.R;

import cz.ceskahospodynka.mobile.core.Settings;
import cz.ceskahospodynka.mobile.core.beans.Menu;
import cz.ceskahospodynka.mobile.core.beans.Recipe;
import cz.ceskahospodynka.mobile.core.tasks.AfterExecutor;
import cz.ceskahospodynka.mobile.core.tasks.RecipeTask;
import cz.ceskahospodynka.mobile.core.utils.AES;

import roboguice.inject.ContentView;
import roboguice.inject.InjectView;

/**
 * Created by tomas on 21.4.14.
 */

@ContentView(R.layout.activity_menu_detail)
public class MenuDetailActivity extends ProcessActivity {

    // sobota
    @InjectView(R.id.sat_lunch_img)
    private ImageView satLunchImg;

    @InjectView(R.id.sat_lunch_txt)
    private TextView satLunchTxt;

    @InjectView(R.id.sat_soup_img)
    private ImageView satSoupImg;

    @InjectView(R.id.sat_soup_txt)
    private TextView satSoupTxt;

    @InjectView(R.id.sat_dinner_img)
    private ImageView satDinnerImg;

    @InjectView(R.id.sat_dinner_txt)
    private TextView satDinnerTxt;

    @InjectView(R.id.sat_salad_img)
    private ImageView satSaladImg;

    @InjectView(R.id.sat_salad_txt)
    private TextView satSaladTxt;

    // nedele
    @InjectView(R.id.sun_lunch_img)
    private ImageView sunLunchImg;

    @InjectView(R.id.sun_lunch_txt)
    private TextView sunLunchTxt;

    @InjectView(R.id.sun_dinner_img)
    private ImageView sunDinnerImg;

    @InjectView(R.id.sun_dinner_txt)
    private TextView sunDinnerTxt;

    @InjectView(R.id.sun_dessert_img)
    private ImageView sunDessetImg;

    @InjectView(R.id.sun_dessert_txt)
    private TextView sunDessertTxt;

    @InjectView(R.id.sun_other_img)
    private ImageView sunOtherImg;

    @InjectView(R.id.sun_other_txt)
    private TextView sunOtherTxt;


    private Menu.Item item;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Bundle bundle = getIntent().getExtras();
        item = gson.fromJson(bundle.getString(MenusActivity.EXTRA_ITEM), Menu.Item.class);

        if(isOnline()) {
            loadRecipes();
        }

        if(detectDevice() != Device.PHONE){
            initCategories(true);
        }
    }

    private void loadRecipes() {
        loadRecipe(item.getPolevka1(), satSoupTxt, satSoupImg);
        loadRecipe(item.getObed1(), satLunchTxt, satLunchImg);
        loadRecipe(item.getVecere1(), satDinnerTxt, satDinnerImg);
        loadRecipe(item.getSalat(), satSaladTxt, satSaladImg);

        loadRecipe(item.getMoucnik(), sunDessertTxt, sunDessetImg);
        loadRecipe(item.getObed2(), sunLunchTxt, sunLunchImg);
        loadRecipe(item.getVecere2(), sunDinnerTxt, sunDinnerImg);
        loadRecipe(item.getJine(), sunOtherTxt, sunOtherImg);

        setProcess(false);
    }

    private void loadRecipe(final Integer recipeId, final TextView recipeName, final ImageView recipeImg) {

        new RecipeTask(recipeId).task(Recipe.Item.class, new AfterExecutor() {
            @Override
            public void work(Object val) {
                if (val != null && val instanceof Recipe.Item) {
                    Recipe.Item recipe = (Recipe.Item) val;

                    recipeName.setText(AES.decrypt(recipe.getNazev()));
                    Picasso.with(MenuDetailActivity.this)
                            .load(String.format(Settings.API_RECIPES_IMAGE, Settings.API_VERSION, Settings.IMAGE_ORIGINAL, recipeId))
                            .placeholder(R.drawable.ic_launcher_bw)
                            .error(R.drawable.ic_launcher)
                            .into(recipeImg);

                    recipeName.setOnClickListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View view) {
                            goToRecipe(recipeId);
                        }
                    });

                    recipeImg.setOnClickListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View view) {
                            goToRecipe(recipeId);
                        }
                    });
                }
            }
        });
    }

    private void goToRecipe(Integer recipeId) {

        Intent intent = new Intent(this, RecipeActivity.class);
        Bundle bundle = new Bundle();
        bundle.putInt(RecipesActivity.EXTRA_RECIPE_ID, recipeId);
        bundle.putString(MenusActivity.EXTRA_ITEM, gson.toJson(item));

        intent.putExtras(bundle);
        startActivity(intent);

    }
}
