package cz.ceskahospodynka.mobile.ui;

import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;

import org.uhk.ceskahospodynka.mobile.R;

import cz.ceskahospodynka.mobile.core.database.core.RS;
import cz.ceskahospodynka.mobile.core.database.core.TableModel;
import cz.ceskahospodynka.mobile.core.database.hospodynka.Creator;
import cz.ceskahospodynka.mobile.core.database.hospodynka.dao.RecipeModel;
import cz.ceskahospodynka.mobile.core.database.hospodynka.handlers.RecipeHandler;

import cz.ceskahospodynka.mobile.ui.adapters.RecipeItem;
import cz.ceskahospodynka.mobile.ui.adapters.RecipesAdapter;

import java.util.LinkedList;
import java.util.List;

import roboguice.inject.ContentView;
import roboguice.inject.InjectView;

/**
 * Created by tomas, Medoro s.r.o. on 6.4.15.
 */

@ContentView(R.layout.activity_my_recipes)
public class MyRecipesActivity extends ProcessActivity {

    @InjectView(R.id.myRecipes)
    ListView myRecipes;

    private RecipesAdapter adapter;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        setShowNotConnectionInfo(false);
        super.onCreate(savedInstanceState);

        adapter = new RecipesAdapter(this, R.layout.recipe_row, R.string.my_recipes_info, new LinkedList<RecipeItem>());
        setRecipes();

        if(detectDevice() != Device.PHONE){
            initCategories(true);
        }
    }

    private void setRecipes() {
        new AsyncTask<Void, Void, List<RecipeItem>>() {

            @Override
            protected List<RecipeItem> doInBackground(Void... params) {
                return getRecipes();
            }

            @Override
            protected void onPreExecute() {
                setProcess(true);
            }

            @Override
            protected void onPostExecute(final List<RecipeItem> recipes) {
                setAdapter(recipes);
                setProcess(false);
            }
        }.execute();
    }

    private List<RecipeItem> getRecipes() {

        RecipeHandler recipeHandler = new RecipeHandler();
        Creator creator = new Creator(this);
        SQLiteDatabase db = creator.getWritableConnection();
        try{
            final List<RecipeItem> items = new LinkedList<RecipeItem>();
            items.add(new RecipeItem());

            recipeHandler.fetch(db, new RS() {
                @Override
                public void fetch(TableModel model) {
                    RecipeItem item = new RecipeItem((RecipeModel)model);
                    items.add(item);
                }
            }, true);
            return items;
        } finally {
            db.close();
        }
    }

    private void setAdapter(final List<RecipeItem> recipes) {
        adapter.addData(recipes);
        myRecipes.setAdapter(adapter);
        myRecipes.setClickable(true);
        myRecipes.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                if(adapter.getData().get(position).getRecipeId() != null){
                    goToRecipe(adapter.getData().get(position).getRecipeId(), adapter.getData().get(position).isFavorite(), position);
                }
            }
        });
        adapter.notifyDataSetChanged();
    }

    private void goToRecipe(Integer recipeId, boolean favorite, int position) {
        Intent intent = new Intent(this, RecipeActivity.class);
        Bundle bundle = new Bundle();
        bundle.putInt(RecipesActivity.EXTRA_RECIPE_ID, recipeId);
        intent.putExtras(bundle);
        startActivity(intent);
    }
}
