package cz.ceskahospodynka.mobile.ui;

import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;

import org.uhk.ceskahospodynka.mobile.R;
import cz.ceskahospodynka.mobile.core.Settings;
import cz.ceskahospodynka.mobile.core.beans.Recipe;
import cz.ceskahospodynka.mobile.core.database.hospodynka.Creator;
import cz.ceskahospodynka.mobile.core.database.hospodynka.RecipeLoader;
import cz.ceskahospodynka.mobile.core.database.hospodynka.handlers.RecipeHandler;
import cz.ceskahospodynka.mobile.core.tasks.AfterExecutor;
import cz.ceskahospodynka.mobile.core.tasks.BestRecipesTask;
import cz.ceskahospodynka.mobile.ui.adapters.RecipeItem;
import cz.ceskahospodynka.mobile.ui.adapters.RecipesAdapter;

import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import roboguice.inject.ContentView;
import roboguice.inject.InjectView;

/**
 * Created with IntelliJ IDEA.
 * User: tomas
 * Date: 13.11.13
 * Time: 16:56
 * To change this template use File | Settings | File Templates.
 */

@ContentView(R.layout.activity_news)
public class NewsActivity extends ProcessActivity {

    private static final String TAG = NewsActivity.class.getSimpleName();

    @InjectView(R.id.newsRecipeList)
    ListView listView;

    private RecipesAdapter adapter;
    private Integer previousRecipePosition;

    private RecipeLoader recipeLoader;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        Bundle bundle = getIntent().getExtras();
        if (bundle != null) {
            previousRecipePosition = bundle.getInt(RecipesActivity.EXTRA_RECIPE_POSITION);
        }

        recipeLoader = new RecipeLoader(this);

        adapter = new RecipesAdapter(NewsActivity.this, R.layout.recipe_row, R.string.news_info, new LinkedList<RecipeItem>());

        if(isOnline()) {
            setRecipes();
        }

        if(detectDevice() != Device.PHONE){
            initCategories(true);
        }
    }

    private void setRecipes() {

        new BestRecipesTask(Settings.API_RECIPES_NEWS, Settings.RECIPES_NEWS).task(Recipe.class, new AfterExecutor() {
            @Override
            public void work(Object val) {

                if (val != null && val instanceof Recipe) {

                    Recipe recipe = (Recipe) val;
                    List<RecipeItem> recipes = new LinkedList<RecipeItem>();


                    Creator creator = new Creator(NewsActivity.this);
                    SQLiteDatabase db = creator.getWritableConnection();

                    try {
                        RecipeHandler recipeHandler = new RecipeHandler();
                        for (Recipe.Item i : recipe.getRecipeList()) {
                            RecipeItem item = recipeLoader.decryptRecipe(i, recipeHandler.isRecipe(db, i.getId()));
                            Log.d(TAG, item.getRecipeId() + ": " + item.getRecipeName());
                            recipes.add(item);
                        }
                    } finally {
                        db.close();
                    }

                    setAdapter(recipes);
                    setProcess(false);

                }
            }
        });
    }

    private void setAdapter(final List<RecipeItem> recipes) {
        adapter.addData(recipes);

        adapter.sort(new Comparator<RecipeItem>() {
            @Override
            public int compare(RecipeItem lhs, RecipeItem rhs) {
                return lhs.getRecipeId() - rhs.getRecipeId();
            }
        });

        listView.setAdapter(adapter);

        if (previousRecipePosition != null) {
            listView.setSelection(previousRecipePosition);
            previousRecipePosition = null;
        }
        listView.setClickable(true);
        listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                if(adapter.getData().get(position).getRecipeId() != null){
                    goToRecipe(adapter.getData().get(position).getRecipeId(), adapter.getData().get(position).isFavorite(), position);
                }
            }
        });
        adapter.notifyDataSetChanged();
    }

    private void goToRecipe(Integer recipeId, boolean favorite, int position) {
        Intent intent = new Intent(this, RecipeActivity.class);
        Bundle bundle = new Bundle();
        bundle.putInt(RecipesActivity.EXTRA_RECIPE_ID, recipeId);
        intent.putExtras(bundle);
        startActivity(intent);
    }
}
