package cz.ceskahospodynka.mobile.ui;

import android.app.ActionBar;
import android.content.Context;
import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;

import org.uhk.ceskahospodynka.mobile.R;
import cz.ceskahospodynka.mobile.core.Settings;
import cz.ceskahospodynka.mobile.core.beans.Recipe;
import cz.ceskahospodynka.mobile.core.database.hospodynka.Creator;
import cz.ceskahospodynka.mobile.core.database.hospodynka.RecipeLoader;
import cz.ceskahospodynka.mobile.core.database.hospodynka.handlers.RecipeHandler;
import cz.ceskahospodynka.mobile.core.tasks.AfterExecutor;
import cz.ceskahospodynka.mobile.core.tasks.RecipesTask;
import cz.ceskahospodynka.mobile.core.utils.CategoryStack;
import cz.ceskahospodynka.mobile.ui.adapters.LoadMoreListView;
import cz.ceskahospodynka.mobile.ui.adapters.RecipeItem;
import cz.ceskahospodynka.mobile.ui.adapters.RecipesAdapter;

import java.util.LinkedList;
import java.util.List;

import roboguice.inject.ContentView;
import roboguice.inject.InjectView;

/**
 * Created with IntelliJ IDEA.
 * User: tomas
 * Date: 14.11.13
 * Time: 19:52
 * To change this template use File | Settings | File Templates.
 */


/**
 * Activita reprezentujici seznam receptu pro danou kategorii
 * Obsahuje slider pro lepsi orientaci mezi recepty (pozor na to, aby slider fungoval, musi byt recepty serazeny podle abecedy!!!)
 */

@ContentView(R.layout.activity_recipes)
public class RecipesActivity extends ProcessActivity implements android.support.v7.app.ActionBar.OnNavigationListener {

    public static final String EXTRA_RECIPE_ID = "recipe_id";
    public static final String EXTRA_RECIPE_POSITION = "recipe_position";
    private static final String ORDER_BY_RECIPE_NAME = "nazev";
    private static final String ORDER_BY_MAIN_INGREDIENT = "hlavs";
    private static final String ORDER_BY_POINTS = "body DESC";

    @InjectView(R.id.list_view)
    LoadMoreListView listView;

    private String[] filter;
    private RecipesAdapter adapter;
    private CategoryStack categoryStack;

    private RecipeLoader recipeLoader;
    private Integer totalCount;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        adapter = new RecipesAdapter(RecipesActivity.this, R.layout.recipe_row, new LinkedList<RecipeItem>());

        Bundle bundle = getIntent().getExtras();
        categoryStack = gson.fromJson(bundle.getString(RecipesCategoriesActivity.EXTRA_CATEGORY_INFO), CategoryStack.class);
        setTitle(categoryStack.getCategoryName());

        filter = getResources().getStringArray(R.array.recipe_sort);
        recipeLoader = new RecipeLoader(this);

        Context context = getSupportActionBar().getThemedContext();
        ArrayAdapter<CharSequence> list = ArrayAdapter.createFromResource(context, R.array.recipe_sort, android.R.layout.simple_spinner_item);
        list.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

        getSupportActionBar().setNavigationMode(ActionBar.NAVIGATION_MODE_LIST);
        getSupportActionBar().setListNavigationCallbacks(list, this);
        getSupportActionBar().setSelectedNavigationItem(categoryStack.getSelectedFilterId());

        if(detectDevice() != Device.PHONE){
            initCategories(true);
        }

        setRecipes();
    }

    private void setRecipes() {
            setProcess(true);
            new RecipesTask(categoryStack.getCategoryName(), categoryStack.getApiOffset(), categoryStack.getApiSize(), getOrderColumnURL(filter[categoryStack.getSelectedFilterId()])).task(Recipe.class, new AfterExecutor() {
                @Override
                public void work(Object val) {
                    if (val != null && val instanceof Recipe) {
                        Recipe recipe = (Recipe) val;

                        totalCount = recipe.getCount();
                        List<RecipeItem> recipes = new LinkedList<RecipeItem>();

                        Creator creator = new Creator(RecipesActivity.this);
                        SQLiteDatabase db = creator.getWritableConnection();

                        try {
                            RecipeHandler recipeHandler = new RecipeHandler();
                            for (Recipe.Item i : recipe.getRecipeList()) {
                                recipes.add(recipeLoader.decryptRecipe(i, recipeHandler.isRecipe(db, i.getId())));
                            }
                        } finally {
                            db.close();
                        }

                        setAdapter(recipes);
                        setProcess(false);
                    }
                }
            });
    }

    private void setAdapter(final List<RecipeItem> recipes) {
        adapter.addData(recipes);
        listView.setAdapter(adapter);

        listView.setClickable(true);
        listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                goToRecipe(adapter.getData().get(position).getRecipeId(), adapter.getData().get(position).isFavorite(), position);
            }
        });
        adapter.notifyDataSetChanged();

        listView.setOnLoadMoreListener(new LoadMoreListView.OnLoadMoreListener() {
            @Override
            public void onLoadMore() {
                setProcess(true);
                categoryStack.setApiOffset(categoryStack.getApiOffset() + Settings.RECIPES_API_STEP);

                new RecipesTask(categoryStack.getCategoryName(), categoryStack.getApiOffset(), categoryStack.getApiSize(), getOrderColumnURL(filter[categoryStack.getSelectedFilterId()])).task(Recipe.class, new AfterExecutor() {
                    @Override
                    public void work(Object val) {
                        if(val != null && val instanceof Recipe){
                            Recipe recipe = (Recipe) val;
                            List<RecipeItem> recipes = new LinkedList<RecipeItem>();
                            Creator creator = new Creator(RecipesActivity.this);
                            SQLiteDatabase db = creator.getWritableConnection();
                            try {
                                RecipeHandler recipeHandler = new RecipeHandler();
                                for (Recipe.Item i : recipe.getRecipeList()) {
                                    recipes.add(recipeLoader.decryptRecipe(i, recipeHandler.isRecipe(db, i.getId())));
                                }
                            } finally {
                                db.close();
                            }

                            adapter.addData(recipes);
                            adapter.notifyDataSetChanged();
                            listView.onLoadMoreComplete();

                            if(totalCount != null && totalCount <= adapter.getCount()) {
                                listView.setOnLoadMoreListener(null);
                            }

                            setProcess(false);
                        }
                    }
                });
            }
        });
    }

    private void goToRecipe(Integer recipeId, boolean favorite, int position) {
        Intent intent = new Intent(this, RecipeActivity.class);
        Bundle bundle = new Bundle();
        bundle.putInt(EXTRA_RECIPE_ID, recipeId);
        intent.putExtras(bundle);
        startActivity(intent);
    }

    private String getOrderColumnURL(String order) {
        if (getResources().getString(R.string.recipes_sort_name).equals(order)) {
            return ORDER_BY_RECIPE_NAME;
        } else if (getResources().getString(R.string.recipes_sort_points).equals(order)) {
            return ORDER_BY_POINTS;
        } else if (getResources().getString(R.string.recipes_sort_main_ingredient).equals(order)) {
            return ORDER_BY_MAIN_INGREDIENT;
        }
        return null;
    }

    @Override
    public boolean onNavigationItemSelected(int itemPosition, long itemId) {
        categoryStack.setSelectedFilterId(itemPosition);
        categoryStack.setApiOffset(0);

        adapter = new RecipesAdapter(RecipesActivity.this, R.layout.recipe_row, new LinkedList<RecipeItem>());
        setRecipes();
        return true;
    }
}
