package cz.ceskahospodynka.mobile.ui;

import android.annotation.TargetApi;
import android.app.ActionBar;
import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.os.Build;
import android.os.Bundle;
import android.view.View;
import android.widget.AdapterView;
import android.widget.GridView;

import org.uhk.ceskahospodynka.mobile.R;

import cz.ceskahospodynka.mobile.core.database.core.RS;
import cz.ceskahospodynka.mobile.core.database.core.TableModel;
import cz.ceskahospodynka.mobile.core.database.hospodynka.Creator;
import cz.ceskahospodynka.mobile.core.database.hospodynka.dao.CategoryModel;
import cz.ceskahospodynka.mobile.core.database.hospodynka.handlers.CategoryHandler;
import cz.ceskahospodynka.mobile.core.tasks.AfterExecutor;
import cz.ceskahospodynka.mobile.core.utils.CategoryStack;
import cz.ceskahospodynka.mobile.ui.adapters.CategoryAdapter;
import cz.ceskahospodynka.mobile.ui.adapters.CategoryGridItem;

import java.util.LinkedList;
import java.util.List;

import roboguice.inject.ContentView;
import roboguice.inject.InjectView;

/**
 * Created with IntelliJ IDEA.
 * User: tomas
 * Date: 7.11.13
 * Time: 21:21
 * To change this template use File | Settings | File Templates.
 */

@ContentView(R.layout.activity_category)
public class RecipesCategoriesActivity extends ProcessActivity {

    public static final String TAG = RecipesCategoriesActivity.class.getSimpleName();
    public static final String EXTRA_CATEGORY_INFO = "cateogry_info";

    @InjectView(R.id.categoryView)
    GridView categoryGrid;

    @TargetApi(Build.VERSION_CODES.HONEYCOMB_MR1)
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if(isOnline()) {
            CategoryHelper categoryHelper = new CategoryHelper(this);
            categoryHelper.syncCategory(new AfterExecutor() {
                @Override
                public void work(Object val) {
                    Creator creator = new Creator(RecipesCategoriesActivity.this);
                    SQLiteDatabase db = creator.getWritableConnection();
                    try {
                        final List<CategoryGridItem> categories = new LinkedList<CategoryGridItem>();

                        CategoryHandler categoryHandler = new CategoryHandler();
                        categoryHandler.fetch(db, new RS() {
                            @Override
                            public void fetch(TableModel model) {
                                categories.add(new CategoryGridItem(((CategoryModel)model).categoryName, ((CategoryModel)model).categoryImage));
                            }
                        });

                        categoryGrid.setAdapter(new CategoryAdapter(RecipesCategoriesActivity.this, categories));
                        categoryGrid.setOnItemClickListener(new AdapterView.OnItemClickListener() {
                            @Override
                            public void onItemClick(AdapterView<?> parent, View v, int position, long id) {
                                goRecipes(categories.get(position).getTitle());
                            }
                        });
                        categoryGrid.getLayoutParams().height = ActionBar.LayoutParams.MATCH_PARENT;
                        setProcess(false);

                    } finally {
                        db.close();
                    }
                }
            });
        } else {
            setProcess(false);
        }
    }

    private void goRecipes(String category) {
        CategoryStack categoryStack = new CategoryStack(category);

        Bundle bundle = new Bundle();
        bundle.putString(EXTRA_CATEGORY_INFO, gson.toJson(categoryStack));

        Intent intent = new Intent(this, RecipesActivity.class);
        intent.putExtras(bundle);
        startActivity(intent);
    }
}
