package cz.ceskahospodynka.mobile.ui;

import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.AdapterView;
import android.widget.EditText;
import android.widget.ImageButton;

import org.uhk.ceskahospodynka.mobile.R;
import cz.ceskahospodynka.mobile.core.Settings;
import cz.ceskahospodynka.mobile.core.beans.Recipe;
import cz.ceskahospodynka.mobile.core.database.hospodynka.Creator;
import cz.ceskahospodynka.mobile.core.database.hospodynka.RecipeLoader;
import cz.ceskahospodynka.mobile.core.database.hospodynka.handlers.RecipeHandler;
import cz.ceskahospodynka.mobile.core.tasks.AfterExecutor;
import cz.ceskahospodynka.mobile.core.tasks.SearchTask;
import cz.ceskahospodynka.mobile.ui.adapters.LoadMoreListView;
import cz.ceskahospodynka.mobile.ui.adapters.RecipeItem;
import cz.ceskahospodynka.mobile.ui.adapters.RecipesAdapter;

import java.util.LinkedList;
import java.util.List;

import roboguice.inject.ContentView;
import roboguice.inject.InjectView;

/**
 * Created by tomas, Medoro s.r.o. on 22.4.15.
 */

@ContentView((R.layout.activity_recipes_search))
public class SearchRecipesActivity extends ProcessActivity {

    public static final String BUNDLE_SEARCH_TEXT = "searchText";

    @InjectView(R.id.search)
    EditText search;

    @InjectView(R.id.search_btn)
    ImageButton searchBtn;

    @InjectView(R.id.recipes)
    LoadMoreListView recipesView;

    private RecipeLoader recipeLoader;
    private RecipesAdapter adapter;
    private Integer start = 0;
    private Integer limit = Settings.RECIPES_API_STEP;
    private Integer totalCount;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        recipeLoader = new RecipeLoader(this);
        adapter = new RecipesAdapter(SearchRecipesActivity.this, R.layout.recipe_row, new LinkedList<RecipeItem>());
        searchBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                loadRecipe(search.getText().toString());
            }
        });

        Bundle searchBundle = getIntent().getExtras();
        if(searchBundle != null) {
            String searchText = searchBundle.getString(BUNDLE_SEARCH_TEXT);
            if (searchText != null && searchText.length() > 0) {
                loadRecipe(searchText);

                search.setText(searchText);
                searchBtn.requestFocus();
            }
        }

        if(detectDevice() != Device.PHONE){
            initCategories(true);
        }
    }

    private void loadRecipe(String searchText){
        setProcess(true);
        adapter.clear();
        setRecipes(searchText);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        setProcess(false);
        return true;
    }

    private void setRecipes(final String searchText) {
            setProcess(true);
            new SearchTask(searchText, start, limit).task(Recipe.class, new AfterExecutor() {
                @Override
                public void work(Object val) {
                    if (val != null && val instanceof Recipe) {
                        Recipe recipe = (Recipe) val;

                        totalCount = recipe.getCount();
                        List<RecipeItem> recipes = new LinkedList<RecipeItem>();

                        Creator creator = new Creator(SearchRecipesActivity.this);
                        SQLiteDatabase db = creator.getWritableConnection();

                        try {
                            RecipeHandler recipeHandler = new RecipeHandler();
                            for (Recipe.Item i : recipe.getRecipeList()) {
                                recipes.add(recipeLoader.decryptRecipe(i, recipeHandler.isRecipe(db, i.getId())));
                            }
                        } finally {
                            db.close();
                        }

                        setAdapter(recipes, searchText);
                        setProcess(false);
                    }
                }
            });

    }

    private void setAdapter(final List<RecipeItem> recipes, final String searchText) {
        adapter.addData(recipes);
        recipesView.setAdapter(adapter);


        recipesView.setClickable(true);
        recipesView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                if(adapter.getData() != null && adapter.getData().size() > position) {
                    goToRecipe(adapter.getData().get(position).getRecipeId(), adapter.getData().get(position).isFavorite(), position);
                }
            }
        });
        adapter.notifyDataSetChanged();

        recipesView.setOnLoadMoreListener(new LoadMoreListView.OnLoadMoreListener() {
            @Override
            public void onLoadMore() {
                setProcess(true);
                start += Settings.RECIPES_API_STEP;
                new SearchTask(searchText, start, limit).task(Recipe.class, new AfterExecutor() {
                    @Override
                    public void work(Object val) {
                        if(val != null && val instanceof Recipe){
                            Recipe recipe = (Recipe) val;
                            List<RecipeItem> recipes = new LinkedList<RecipeItem>();
                            Creator creator = new Creator(SearchRecipesActivity.this);
                            SQLiteDatabase db = creator.getWritableConnection();
                            try {
                                RecipeHandler recipeHandler = new RecipeHandler();
                                for (Recipe.Item i : recipe.getRecipeList()) {
                                    recipes.add(recipeLoader.decryptRecipe(i, recipeHandler.isRecipe(db, i.getId())));
                                }
                            } finally {
                                db.close();
                            }

                            adapter.addData(recipes);
                            adapter.notifyDataSetChanged();

                            Log.d("TEST", "total count: " + totalCount);
                            Log.d("TEST", "adapter count: " + adapter.getCount());

                            if(totalCount == null || totalCount > adapter.getCount()) {
                                recipesView.setOnLoadMoreListener(null);
                            } else {
                                recipesView.onLoadMoreComplete();
                            }

                            setProcess(false);
                        }
                    }
                });
            }
        });
    }

    private void goToRecipe(Integer recipeId, boolean favorite, int position) {
        Intent intent = new Intent(this, RecipeActivity.class);
        Bundle bundle = new Bundle();
        bundle.putInt(RecipesActivity.EXTRA_RECIPE_ID, recipeId);
        intent.putExtras(bundle);
        startActivity(intent);
    }
}
