package cz.ceskahospodynka.mobile.ui.adapters;

import android.app.Activity;
import android.content.Context;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import com.squareup.picasso.Picasso;

import org.uhk.ceskahospodynka.mobile.R;

import java.util.ArrayList;
import java.util.List;

/**
 * Created with IntelliJ IDEA.
 * User: tomas
 * Date: 15.11.13
 * Time: 23:12
 * To change this template use File | Settings | File Templates.
 */
public class RecipesAdapter extends ArrayAdapter<RecipeItem> {

    private int layoutResourceId, infoTextId;
    private List<RecipeItem> data = new ArrayList<RecipeItem>();

    public RecipesAdapter(Context context, int layoutResourceId, List<RecipeItem> data) {
        this(context, layoutResourceId, 0, data);
    }

    public RecipesAdapter(Context context, int layoutResourceId,int infoTextId, List<RecipeItem> data) {
        super(context, layoutResourceId, data);
        this.layoutResourceId = layoutResourceId;
        this.infoTextId = infoTextId;
        this.data = data;
    }

    @Override
    public View getView(final int position, View convertView, ViewGroup parent) {

        LayoutInflater inflater = ((Activity) getContext()).getLayoutInflater();
        View row = null;

        if(data.get(position).getRecipeId() == null){
            row = inflater.inflate(R.layout.view_info, parent, false);
            TextView infoText = (TextView) row.findViewById(R.id.info_text);
            infoText.setText(infoTextId);
        } else {
            row = inflater.inflate(layoutResourceId, parent, false);
            RecordHolder  holder = new RecordHolder();
            holder.recipeImage = (ImageView) row.findViewById(R.id.recipe_item_image);
            holder.favorite = (ImageView) row.findViewById(R.id.recipe_favorite);
            holder.title = (TextView) row.findViewById(R.id.recipe_item_text);
            holder.mainIngredient = (TextView) row.findViewById(R.id.recipe_main_ingredient);
            holder.points = (TextView) row.findViewById(R.id.recipe_points);
            row.setTag(holder);

            final RecipeItem item = data.get(position);

            if (item.isFavorite()) {
                holder.favorite.setImageResource(android.R.drawable.ic_menu_save);
            } else {
                holder.favorite.setImageDrawable(null);
            }

            holder.title.setText(item.getRecipeName());
            holder.mainIngredient.setText(item.getMainIngredient());
            holder.points.setText("" + item.getPoints());

            if (item.getBitmapImage() != null) {
                holder.recipeImage.setImageBitmap(item.getBitmapImage());
            } else {

                Picasso.with(getContext())
                    .load(getItem(position).getRecipeImage())
                    .placeholder(R.drawable.ic_launcher_bw)
                    .error(R.drawable.ic_launcher)
                    .into(holder.recipeImage);
            }

        }
        return row;

    }

    public List<RecipeItem> getData() {
        return data;
    }

    public void addData(List<RecipeItem> data) {
        this.data.addAll(data);
    }

    static class RecordHolder {
        ImageView recipeImage;
        ImageView favorite;
        TextView title;
        TextView mainIngredient;
        TextView points;
    }
}
