package cz.ceskahospodynka.mobile.ui.fragments;

import android.app.ActionBar;
import android.content.Intent;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.squareup.picasso.Picasso;

import org.uhk.ceskahospodynka.mobile.R;
import cz.ceskahospodynka.mobile.core.Settings;
import cz.ceskahospodynka.mobile.core.beans.Recipe;
import cz.ceskahospodynka.mobile.core.database.hospodynka.RecipeLoader;
import cz.ceskahospodynka.mobile.core.tasks.AfterExecutor;
import cz.ceskahospodynka.mobile.core.tasks.BestRecipesTask;
import cz.ceskahospodynka.mobile.ui.RecipeActivity;
import cz.ceskahospodynka.mobile.ui.RecipesActivity;
import cz.ceskahospodynka.mobile.ui.adapters.RecipeItem;

import java.util.LinkedList;
import java.util.List;

import roboguice.fragment.RoboFragment;
import roboguice.inject.InjectView;

/**
 * Created by tomas on 21.4.14.
 */
public abstract class LastThreeFragment extends RoboFragment {

    @InjectView(R.id.layout)
    private LinearLayout layout;

    @InjectView(R.id.favorite_image_1)
    private ImageView favoriteImage1;

    @InjectView(R.id.favorite_name_1)
    private TextView favoriteName1;

    @InjectView(R.id.favorite_image_2)
    private ImageView favoriteImage2;

    @InjectView(R.id.favorite_name_2)
    private TextView favoriteName2;

    @InjectView(R.id.favorite_image_3)
    private ImageView favoriteImage3;

    @InjectView(R.id.favorite_name_3)
    private TextView favoriteName3;

    @InjectView(R.id.favorite_empty)
    private TextView favoriteEmptyText;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        return inflater.inflate(R.layout.favorite_fragment, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
    }

    @Override
    public void onActivityCreated(@Nullable Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        setRecipes();
    }

    protected void setRecipes(){
        new BestRecipesTask(getAddress(), Settings.RECIPES_HOMEPAGE).task(Recipe.class, new AfterExecutor() {
            @Override
            public void work(Object val) {

                if(val != null && val instanceof Recipe){
                    RecipeLoader recipeLoader = new RecipeLoader(getActivity());
                    List<Data> recipes = new LinkedList<Data>();
                    for(Recipe.Item i : ((Recipe)val).getRecipeList()){
                        RecipeItem recipe = recipeLoader.decryptRecipe(i, false);
                        recipes.add(new Data(recipe.getRecipeName(), recipe.getRecipeImage(), recipe.getRecipeId()));

                    }
                    setData(recipes);
                }
            }
        });
    }

    protected void setData(List<Data> data) {

        favoriteImage1.getLayoutParams().height = 0;
        favoriteImage2.getLayoutParams().height = 0;
        favoriteImage3.getLayoutParams().height = 0;
        favoriteName1.getLayoutParams().height = 0;
        favoriteName2.getLayoutParams().height = 0;
        favoriteName3.getLayoutParams().height = 0;
        favoriteEmptyText.getLayoutParams().height = 0;

        favoriteImage1.setImageBitmap(null);
        favoriteImage2.setImageBitmap(null);
        favoriteImage3.setImageBitmap(null);
        favoriteName1.setText("");
        favoriteName2.setText("");
        favoriteName3.setText("");

        if (data.isEmpty()) {
            favoriteEmptyText.getLayoutParams().height = ActionBar.LayoutParams.WRAP_CONTENT;
            favoriteEmptyText.setText(R.string.recipe_favrite_empty);
            return;
        }

        favoriteImage1.getLayoutParams().height = 80;
        favoriteImage2.getLayoutParams().height = 80;
        favoriteImage3.getLayoutParams().height = 80;
        favoriteName1.getLayoutParams().height = ActionBar.LayoutParams.WRAP_CONTENT;
        favoriteName2.getLayoutParams().height = ActionBar.LayoutParams.WRAP_CONTENT;
        favoriteName3.getLayoutParams().height = ActionBar.LayoutParams.WRAP_CONTENT;

        int pos = 0;
        if (data.size() > pos) {
            setImage(data.get(pos), favoriteImage1);
            favoriteName1.setText(data.get(pos).getRecipeName());
            setOnClick(favoriteImage1, favoriteName1, data.get(pos).getRecipeId());
        } else {
            favoriteName1.getLayoutParams().height = 0;
            favoriteName2.getLayoutParams().height = 0;
            favoriteName3.getLayoutParams().height = 0;
        }

        pos++;
        if (data.size() > pos) {
            setImage(data.get(pos), favoriteImage2);
            favoriteName2.setText(data.get(pos).getRecipeName());
            setOnClick(favoriteImage2, favoriteName2, data.get(pos).getRecipeId());
        } else {
            favoriteName2.getLayoutParams().height = 0;
            favoriteName3.getLayoutParams().height = 0;
        }

        pos++;
        if (data.size() > pos) {
            setImage(data.get(pos), favoriteImage3);
            favoriteName3.setText(data.get(pos).getRecipeName());
            setOnClick(favoriteImage3, favoriteName3, data.get(pos).getRecipeId());
        } else {
            favoriteName3.getLayoutParams().height = 0;
        }
    }

    private void setOnClick(ImageView image, TextView text, final Integer id) {
        image.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                goToRecipe(id);
            }
        });

        text.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                goToRecipe(id);
            }
        });
    }

    private void setImage(Data data, ImageView component){
        if(data.getImage() == null) {
            Picasso.with(getActivity())
                    .load(data.getRecipeImageUrl())
                    .placeholder(R.drawable.ic_launcher_bw)
                    .error(R.drawable.ic_launcher)
                    .into(component);
        } else {
            component.setImageBitmap(data.getImage());
        }
    }

    protected void goToRecipeFromMain(Integer recipeId) {
        Intent intent = new Intent(getActivity(), RecipeActivity.class);
        Bundle bundle = new Bundle();
        bundle.putInt(RecipesActivity.EXTRA_RECIPE_ID, recipeId);

        intent.putExtras(bundle);
        startActivity(intent);
    }

    protected abstract String getAddress();

    protected abstract void goToRecipe(Integer recipeId);

    public class Data {

        private String recipeName;
        private String recipeImageUrl;
        private Integer recipeId;
        private Bitmap image;

        public Data(String recipeName, String recipeImageUrl, Integer recipeId) {
            this(recipeName, recipeImageUrl, recipeId, null);
        }

        public Data(String recipeName, String recipeImageUrl, Integer recipeId, Bitmap image) {
            this.recipeName = recipeName;
            this.recipeImageUrl = recipeImageUrl;
            this.recipeId = recipeId;
            this.image = image;
        }

        public String getRecipeName() {
            return recipeName;
        }

        public String getRecipeImageUrl() {
            return recipeImageUrl;
        }

        public Integer getRecipeId() {
            return recipeId;
        }

        public Bitmap getImage() {
            return image;
        }
    }

}
