package cz.ceskahospodynka.mobile.ui.fragments;

import android.app.ActionBar;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ImageView;
import android.widget.TextView;

import com.squareup.picasso.Picasso;

import org.lucasr.twowayview.TwoWayView;
import org.uhk.ceskahospodynka.mobile.R;

import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import cz.ceskahospodynka.mobile.core.Settings;
import cz.ceskahospodynka.mobile.core.beans.Recipe;
import cz.ceskahospodynka.mobile.core.database.hospodynka.RecipeLoader;
import cz.ceskahospodynka.mobile.core.tasks.AfterExecutor;
import cz.ceskahospodynka.mobile.core.tasks.BestRecipesTask;
import cz.ceskahospodynka.mobile.ui.RecipeActivity;
import cz.ceskahospodynka.mobile.ui.RecipesActivity;
import cz.ceskahospodynka.mobile.ui.adapters.HorizontalAdapter;
import cz.ceskahospodynka.mobile.ui.adapters.RecipeItem;
import roboguice.fragment.RoboFragment;
import roboguice.inject.InjectView;

/**
 * Created by tomas, Medoro s.r.o. on 6.5.15.
 */
public abstract class LastThreeHorizontalFragment extends RoboFragment {

    @InjectView(R.id.favorite_empty)
    TextView favoriteEmptyText;

    @InjectView(R.id.horizontal_view)
    TwoWayView horizontalView;

    private HorizontalAdapter horizontalAdapter;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        return inflater.inflate(R.layout.favorite_horizontal_fragment, container, false);
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
    }

    @Override
    public void onActivityCreated(@Nullable Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        initRecipes(getActivity());

        horizontalView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {
                goToRecipeFromMain(getActivity(), horizontalAdapter.getItem(i).getRecipeId());
            }
        });
    }

    protected void initRecipes(Context context){
        setRecipes(context);
    }

    public void setRecipes(final Context context){

        if(getAddress() == null){
            return;
        }

        new BestRecipesTask(getAddress(), getLimit()).task(Recipe.class, new AfterExecutor() {
            @Override
            public void work(Object val) {

                if(val != null && val instanceof Recipe){
                    RecipeLoader recipeLoader = new RecipeLoader(context);
                    List<Data> recipes = new LinkedList<Data>();
                    for(Recipe.Item i : ((Recipe)val).getRecipeList()){
                        RecipeItem recipe = recipeLoader.decryptRecipe(i, false);
                        recipes.add(new Data(recipe.getRecipeName(), recipe.getRecipeImage(), recipe.getRecipeId(), null, recipe.getPoints()));

                    }
                    setData(context, recipes);
                }
            }
        });
    }

    protected int getLimit(){
        return Settings.RECIPES_HOMEPAGE;
    }

    protected void setData(Context context, List<Data> data) {

        horizontalView.getLayoutParams().height = 0;

        if (data.isEmpty()) {
            favoriteEmptyText.getLayoutParams().height = ActionBar.LayoutParams.WRAP_CONTENT;
            favoriteEmptyText.setText(R.string.recipe_favrite_empty);
            return;
        }

        favoriteEmptyText.getLayoutParams().height = 0;
        horizontalView.getLayoutParams().height = 80;

        horizontalAdapter = new HorizontalAdapter(context, data);
        horizontalView.setAdapter(horizontalAdapter);

        sortAdapter(horizontalAdapter);
        horizontalAdapter.notifyDataSetChanged();
    }

    protected void sortAdapter(HorizontalAdapter adapter){
    }

    protected void goToRecipeFromMain(Context context, Integer recipeId) {
        Intent intent = new Intent(context, RecipeActivity.class);
        Bundle bundle = new Bundle();
        bundle.putInt(RecipesActivity.EXTRA_RECIPE_ID, recipeId);

        intent.putExtras(bundle);
        startActivity(intent);
    }

    protected abstract String getAddress();

    //protected abstract void goToRecipe(Integer recipeId);

    public class Data {

        private String recipeName;
        private String recipeImageUrl;
        private Integer recipeId;
        private Bitmap image;

        private Integer points;


        public Data(String recipeName, String recipeImageUrl, Integer recipeId, Bitmap image, Integer points) {
            this.recipeName = recipeName;
            this.recipeImageUrl = recipeImageUrl;
            this.recipeId = recipeId;
            this.image = image;
            this.points = points;
        }

        public String getRecipeName() {
            return recipeName;
        }

        public String getRecipeImageUrl() {
            return recipeImageUrl;
        }

        public Integer getRecipeId() {
            return recipeId;
        }

        public Bitmap getImage() {
            return image;
        }

        public Integer getPoints() {
            return points;
        }
    }
}
