package cz.ceskahospodynka.mobile.ui.tabs;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ListView;

import org.uhk.ceskahospodynka.mobile.R;

import cz.ceskahospodynka.mobile.core.beans.Recipe;
import cz.ceskahospodynka.mobile.core.database.hospodynka.Creator;
import cz.ceskahospodynka.mobile.core.database.hospodynka.RecipeLoader;
import cz.ceskahospodynka.mobile.core.database.hospodynka.handlers.RecipeHandler;
import cz.ceskahospodynka.mobile.core.tasks.AfterExecutor;
import cz.ceskahospodynka.mobile.core.tasks.BestRecipesTask;

import cz.ceskahospodynka.mobile.ui.BestRecipesActivity;
import cz.ceskahospodynka.mobile.ui.ProcessActivity;
import cz.ceskahospodynka.mobile.ui.RecipeActivity;
import cz.ceskahospodynka.mobile.ui.RecipesActivity;
import cz.ceskahospodynka.mobile.ui.adapters.BestRecipeSectionAdapter;
import cz.ceskahospodynka.mobile.ui.adapters.RecipeItem;
import cz.ceskahospodynka.mobile.ui.adapters.RecipesAdapter;

import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;

import roboguice.fragment.RoboFragment;

/**
 * Created by tomas on 21.4.14.
 */
public class BestFragment extends RoboFragment {

    protected Integer lastSelectedRecipe;
    protected Integer lastSelectedTab;
    private RecipeLoader recipeLoader;
    private RecipesAdapter adapter;

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
    }

    @Override
    public void onActivityCreated(@Nullable Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        setProperties(getActivity());
        recipeLoader = new RecipeLoader(getActivity());
    }

    protected void setProperties(Activity context) {
        Bundle bundle = context.getIntent().getExtras();

        if (bundle != null) {
            lastSelectedRecipe = bundle.getInt(RecipesActivity.EXTRA_RECIPE_POSITION);
            lastSelectedTab = bundle.getInt(BestRecipesActivity.SELECTED_TAB_INDEX);
        }
    }

    protected void setRecipes(final Context context, final String url, Integer limit, final int infoTextId, final ListView listView) {

        new BestRecipesTask(url, limit).task(Recipe.class, new AfterExecutor() {
            @Override
            public void work(Object val) {

                if(val != null && val instanceof Recipe){
                    Recipe recipe = (Recipe) val;
                    List<RecipeItem> recipes = new LinkedList<RecipeItem>();
                    Creator creator = new Creator(context);
                    SQLiteDatabase db = creator.getWritableConnection();

                    try {
                        RecipeHandler recipeHandler = new RecipeHandler();
                        for (Recipe.Item i : recipe.getRecipeList()) {
                            recipes.add(recipeLoader.decryptRecipe(i, recipeHandler.isRecipe(db, i.getId())));
                        }
                    } finally {
                        db.close();
                    }
                    setAdapter(context, listView, infoTextId, recipes);
                    if(context instanceof ProcessActivity){
                        ((ProcessActivity)context).setProcess(false);
                    }
                }
            }
        });
    }

    private void setAdapter(Context context, ListView recipesView, int infoTextId, final List<RecipeItem> recipes) {
        adapter = new RecipesAdapter(context, R.layout.recipe_row, infoTextId, new LinkedList<RecipeItem>());
        adapter.addData(recipes);
        recipesView.setAdapter(adapter);

        if (lastSelectedRecipe != null) {
            recipesView.setSelection(lastSelectedRecipe);
            lastSelectedRecipe = null;
        }
        recipesView.setClickable(true);
        recipesView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                if(adapter.getData().get(position).getRecipeId() != null){
                    goToRecipe(adapter.getData().get(position).getRecipeId());
                }
            }
        });

        adapter.sort(new Comparator<RecipeItem>() {
            @Override
            public int compare(RecipeItem lhs, RecipeItem rhs) {
                return rhs.getPoints() - lhs.getPoints();
            }
        });

        adapter.notifyDataSetChanged();
    }

    private void goToRecipe(Integer recipeId) {
        Intent intent = new Intent(getActivity(), RecipeActivity.class);
        Bundle bundle = new Bundle();
        bundle.putInt(RecipesActivity.EXTRA_RECIPE_ID, recipeId);
        bundle.putInt(BestRecipesActivity.SELECTED_TAB_INDEX, this instanceof BestPointsTab ? BestRecipeSectionAdapter.TAB_POINTS : BestRecipeSectionAdapter.TAB_TOPWATCH);
        intent.putExtras(bundle);
        startActivity(intent);
    }
}
