package cz.ceskahospodynka.mobile.ui.tabs;

import android.app.Activity;
import android.app.Dialog;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.text.Html;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.squareup.picasso.Picasso;

import org.uhk.ceskahospodynka.mobile.R;
import cz.ceskahospodynka.mobile.core.Settings;
import cz.ceskahospodynka.mobile.core.beans.DefaultApi;
import cz.ceskahospodynka.mobile.core.beans.Recipe;
import cz.ceskahospodynka.mobile.core.database.hospodynka.Creator;
import cz.ceskahospodynka.mobile.core.database.hospodynka.dao.PointDAO;
import cz.ceskahospodynka.mobile.core.database.hospodynka.dao.RecipeModel;
import cz.ceskahospodynka.mobile.core.database.hospodynka.handlers.PointHandler;
import cz.ceskahospodynka.mobile.core.database.hospodynka.handlers.RecipeHandler;
import cz.ceskahospodynka.mobile.core.tasks.AddPointTask;
import cz.ceskahospodynka.mobile.core.tasks.AfterExecutor;
import cz.ceskahospodynka.mobile.core.tasks.RecipeTask;
import cz.ceskahospodynka.mobile.core.tasks.SaveFavoriteTask;
import cz.ceskahospodynka.mobile.core.utils.AES;
import cz.ceskahospodynka.mobile.core.utils.IO;
import cz.ceskahospodynka.mobile.ui.BaseActivity;

import cz.ceskahospodynka.mobile.ui.RecipeActivity;
import roboguice.inject.InjectView;
import uk.co.senab.photoview.PhotoViewAttacher;

/**
 * Created with IntelliJ IDEA.
 * User: tomas
 * Date: 14.11.13
 * Time: 19:52
 * To change this template use File | Settings | File Templates.
 */

public class RecipeMainTab extends RecipeTab {

    public static final String TAG = "RECIPE";

    @InjectView(R.id.recipe_name)
    private TextView recipeName;

    @InjectView(R.id.recipe_image)
    private ImageView recipeImage;

    @InjectView(R.id.recipe_ingredient)
    private TextView recipeIngredient;

    @InjectView(R.id.recipe_advance)
    private TextView recipeAdvance;

    @InjectView(R.id.favorite)
    private Button favoriteImageButton;

    @InjectView(R.id.share)
    private Button shareImageButton;

    @InjectView(R.id.point)
    private Button pointImageButton;

    private String category;
    private String mainIngredience;
    private Integer points;
    private Bitmap imageBitmap;
    private PhotoViewAttacher attacher;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.recipe_main, container, false);
        return rootView;
    }

    @Override
    public void onViewCreated(View view, Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
    }

    @Override
    public void onActivityCreated(@Nullable Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);

        setProperties(getActivity());
        if(favoriteRecipe) {

            RecipeModel recipe;
            RecipeHandler recipeHandler = new RecipeHandler();
            Creator creator = new Creator(getActivity());

            SQLiteDatabase db = creator.getReadableConnection();
            try {
                recipe = recipeHandler.findRecipe(db, recipeId);
            } finally {
                db.close();
            }

            recipeName.setText(recipe.recipeName);
            if (recipe.recipeIngredient != null) {
                recipeIngredient.setText(Html.fromHtml(recipe.recipeIngredient));
            }
            if (recipe.recipeAdvance != null) {
                recipeAdvance.setText(Html.fromHtml(recipe.recipeAdvance));
            }
            category = recipe.recipeCategory;
            imageBitmap = recipe.recipeImage;
            recipeImage.setImageBitmap(imageBitmap);

            favoriteImageButton.setText(R.string.recipe_remove);

        } else {
            loadRecipeFromUrl(getActivity());

            Picasso.with(getActivity())
                    .load(String.format(Settings.API_RECIPES_IMAGE, Settings.API_VERSION, Settings.IMAGE_ORIGINAL, recipeId))
                    .placeholder(R.drawable.ic_launcher_bw)
                    .error(R.drawable.ic_launcher)
                    .into(recipeImage);
        }
        addListenerOnButton(getActivity());
        checkPointed(getActivity());
    }

    private void checkPointed(Context context){
        PointHandler pointHandler = new PointHandler();
        Creator creator = new Creator(context);

        SQLiteDatabase db = creator.getReadableConnection();
        try {
            if(pointHandler.isPointed(db, recipeId)){
                pointImageButton.setEnabled(false);
                pointImageButton.setText(R.string.recipe_pointed_btn);
            } else {
                pointImageButton.setEnabled(true);
                pointImageButton.setText(R.string.recipe_point);
            }
        } finally {
            db.close();
        }
    }

    private void loadRecipeFromUrl(final Context context) {

        favoriteImageButton.setEnabled(false);
        new RecipeTask(recipeId).task(Recipe.Item.class, new AfterExecutor() {
            @Override
            public void work(Object val) {
                if(val != null && val instanceof Recipe.Item){

                    Recipe.Item recipe = (Recipe.Item) val;

                    recipeName.setText(AES.decrypt(recipe.getNazev()));

                    if (recipe.getSuroviny() != null) {
                        recipeIngredient.setText(Html.fromHtml(AES.decrypt(recipe.getSuroviny())));
                    }
                    if (recipe.getPostup() != null) {
                        recipeAdvance.setText(Html.fromHtml(AES.decrypt(recipe.getPostup())));
                    }

                    mainIngredience = recipe.getHlavs();
                    points = recipe.getBody();

                    category = recipe.getKategorie();

                    if(context instanceof RecipeActivity){
                        ((RecipeActivity) context).renameOpinionTab(((RecipeActivity) context).getString(R.string.recipe_tab_opinion_count, recipe.getOpinions()));
                    }

                    favoriteImageButton.setEnabled(true);
                }
            }
        });
    }

    public void addListenerOnButton(final Context context) {
        favoriteImageButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View arg0) {
                saveFavorite(context);
            }
        });

        shareImageButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View arg0) {
                shareRecipe(context);
            }
        });

        pointImageButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View arg0) {
                addPoint(context);
            }
        });

        recipeImage.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                showImageDialog(context);
            }
        });
    }

    private void showImageDialog(Context context) {

        final Dialog dialog = new Dialog(context);
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE);
        dialog.getWindow().setBackgroundDrawable(new ColorDrawable(android.graphics.Color.TRANSPARENT));
        dialog.setContentView(R.layout.image_dialog);


        ImageView image = (ImageView) dialog.findViewById(R.id.dialog_image);

        attacher = new PhotoViewAttacher(image);

        if(favoriteRecipe) {
            image.setImageBitmap(imageBitmap);
        } else {
            Picasso.with(context)
                    .load(String.format(Settings.API_RECIPES_IMAGE, Settings.API_VERSION, Settings.IMAGE_ORIGINAL, recipeId))
                    .placeholder(R.drawable.ic_launcher_bw)
                    .error(R.drawable.ic_launcher)
                    .into(image);
        }

        image.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dialog.dismiss();
            }
        });
        dialog.show();
    }

    private void saveFavorite(final Context context) {

        boolean save;
        if(favoriteRecipe){
            // mazani
            RecipeHandler recipeHandler = new RecipeHandler();
            Creator creator = new Creator(context);
            SQLiteDatabase db = creator.getWritableConnection();
            try {
                if(recipeHandler.dropRecipe(db, recipeId)){
                    Toast.makeText(context, getResources().getString(R.string.recipe_favrite_removed), Toast.LENGTH_SHORT).show();
                    favoriteImageButton.setText(R.string.recipe_save);
                    favoriteRecipe = false;
                }
            } finally {
                db.close();
            }
            save = false;
        } else {
            // ukladani

            RecipeHandler recipeHandler = new RecipeHandler();
            Creator creator = new Creator(context);
            SQLiteDatabase db = creator.getWritableConnection();
            try {

                if (recipeHandler.isRecipe(db, recipeId)) {
                    Toast.makeText(context, getResources().getString(R.string.recipe_favrited), Toast.LENGTH_SHORT).show();
                    return;
                }

                ContentValues values = new ContentValues();
                values.put(((RecipeModel) recipeHandler.getModel()).getRecipeIdColumn(), recipeId);
                values.put(((RecipeModel) recipeHandler.getModel()).getRecipeNameColumn(), recipeName.getText().toString());
                values.put(((RecipeModel) recipeHandler.getModel()).getRecipeIngredientColumn(), recipeIngredient.getText().toString());
                values.put(((RecipeModel) recipeHandler.getModel()).getRecipeAdvanceColumn(), recipeAdvance.getText().toString());
                values.put(((RecipeModel) recipeHandler.getModel()).getRecipeCategoryColumn(), category);
                values.put(((RecipeModel) recipeHandler.getModel()).getRecipeMainIngredientColumn(), AES.decrypt(mainIngredience));
                values.put(((RecipeModel) recipeHandler.getModel()).getRecipePointsColumn(), points);
                values.put(((RecipeModel) recipeHandler.getModel()).getRecipeImageColumn(), IO.getBitmapAsByteArray(((BitmapDrawable) recipeImage.getDrawable()).getBitmap()));
                db.insert(recipeHandler.getGenerator().getTableName(), null, values);
            } finally {
                db.close();
            }
            Toast.makeText(context, getResources().getString(R.string.recipe_favrite_add), Toast.LENGTH_SHORT).show();
            favoriteImageButton.setText(R.string.recipe_remove);
            favoriteRecipe = true;
            save = true;
        }

        String userId = null;
        if (context instanceof BaseActivity) {

            if(((BaseActivity) context).getString(BaseActivity.PREF_CH_USERNAME) != null){
                userId = ((BaseActivity) context).getString(BaseActivity.PREF_CH_USERNAME);
            } else if(((BaseActivity) context).getString(BaseActivity.PREF_FACEBOOK_USER_UID) != null){
                userId = ((BaseActivity) context).getString(BaseActivity.PREF_FACEBOOK_USER_UID);
            } else if(((BaseActivity) context).getString(BaseActivity.PREF_GOOGLE_USER_UID) != null){
                userId = ((BaseActivity) context).getString(BaseActivity.PREF_GOOGLE_USER_UID);
            }
        }

        if (userId != null) {
            new SaveFavoriteTask(userId, recipeId, save).task(DefaultApi.class, new AfterExecutor() {
                @Override
                public void work(Object val) {
                    if (val != null && val instanceof DefaultApi) {
                        Toast.makeText(context, R.string.recipe_server_save, Toast.LENGTH_SHORT).show();
                    }
                }
            });
        }

    }

    private void shareRecipe(Context context) {
        Intent sharingIntent = new Intent(android.content.Intent.ACTION_SEND);
        sharingIntent.setType("text/plain");

        String shareBody = context.getString(R.string.recipe_share_body, recipeName.getText(), String.format(Settings.WEB_RECIPE, recipeId));
        sharingIntent.putExtra(android.content.Intent.EXTRA_SUBJECT, context.getString(R.string.recipe_share_title));
        sharingIntent.putExtra(android.content.Intent.EXTRA_TEXT, shareBody);
        startActivity(Intent.createChooser(sharingIntent, getString(R.string.recipe_share_dg_title)));
    }

    private void addPoint(final Context context) {
        pointImageButton.setEnabled(false);
        new AddPointTask(recipeId).task(DefaultApi.class, new AfterExecutor() {
            @Override
            public void work(Object val) {
                if(val != null && val instanceof DefaultApi){

                    PointHandler pointHandler = new PointHandler();
                    Creator creator = new Creator(context);
                    SQLiteDatabase db = creator.getWritableConnection();
                    try {
                        ContentValues values = new ContentValues();
                        values.put(PointDAO.RECIPE_COLUMN, recipeId);
                        db.insert(pointHandler.getGenerator().getTableName(), null, values);
                    } finally {
                        db.close();
                    }

                    Toast.makeText(context, R.string.recipe_pointed, Toast.LENGTH_SHORT).show();
                    pointImageButton.setText(R.string.recipe_pointed_btn);
                }
            }
        });
    }
}
