package cz.ceskahospodynka.api.controller;

import java.io.IOException;

import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

@Controller
@RequestMapping(value = "/recipes", method = RequestMethod.POST, produces = "text/plain;charset=UTF-8")
public class RecipesController extends BaseController {

	@RequestMapping(value="/get")
	@ResponseBody
	public String getRecipes(
			@RequestParam("api") String api,
			@RequestParam("category") String category,
			@RequestParam("start") Integer start,
			@RequestParam("limit") Integer limit,
			@RequestParam("order") String order){
		
		return getApi(api).getRecipes(category, start, limit, order);
	}
	
	@RequestMapping(value="/search")
	@ResponseBody
	public String searchRecipes(
			@RequestParam("api") String api,
			@RequestParam("search") String searchString,
			@RequestParam("start") Integer start,
			@RequestParam("limit") Integer limit
			){
		
		return getApi(api).searchRecipes(searchString, start, limit);
	}
	
	@RequestMapping(value="/get-recipe")
	@ResponseBody
	public String getRecipe(
			@RequestParam("api") String api,
			@RequestParam("id") Integer id){

		return getApi(api).getRecipe(id);
	}
	
	@RequestMapping("/news")
	@ResponseBody
	public String getNews(
			@RequestParam("api") String api,
			@RequestParam("limit") Integer limit){

		return getApi(api).getNewRecipes(limit);
	}
	
	@RequestMapping(value="/topwatch")
	@ResponseBody
	public String getTopwatch(
			@RequestParam("api") String api,
			@RequestParam("limit") Integer limit){
		
		return getApi(api).getTopwatchRecipe(limit);
	}
	
	@RequestMapping(value="/bests")
	@ResponseBody
	public String getBests(
			@RequestParam("api") String api,
			@RequestParam("limit") Integer limit){
		
		return getApi(api).getBestRecipes(limit);
	}
	
	@RequestMapping(value="/image/api/{api}/type/{type}/id/{id}", method = RequestMethod.GET)
	public ResponseEntity<byte[]> getImage(
			@PathVariable String api, 
			@PathVariable String type, 
			@PathVariable Integer id) throws IOException{
		return getApi(api).getRecipeImage(type, id);	
	}
	
	@RequestMapping(value="/save-favorite")
	@ResponseBody
	public String saveFavorite(
			@RequestParam("api") String api,
			@RequestParam("user") String user,
			@RequestParam("id") Integer recipeId,
			@RequestParam("save") Boolean save){

		if(save){
			return getApi(api).saveFavoriteRecipe(user, recipeId);
		} else {
			return getApi(api).dropFavoriteRecipe(user, recipeId);
		}
		
	}
	
	@RequestMapping(value="/save-point")
	@ResponseBody
	public String savePoint(
			@RequestParam("api") String api,
			@RequestParam("id") Integer recipeId){

		return getApi(api).savePoint(recipeId);
	}
}
