package cz.ceskahospodynka.api.controller;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.brickred.socialauth.AuthProvider;
import org.brickred.socialauth.Contact;
import org.brickred.socialauth.SocialAuthManager;
import org.brickred.socialauth.spring.bean.SocialAuthTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

import cz.ceskahospodynka.api.pojo.SyncData;
import cz.ceskahospodynka.api.utility.SHA1;
import cz.ceskahospodynka.api.version.Api;

@Controller
public class SuccessController extends BaseController {

	private static final String PROVIDER_FACEBOOK = "facebook";
	private static final String PROVIDER_GOOGLE = "googleplus";
	
	@Autowired
	private SocialAuthTemplate socialAuthTemplate;

	@RequestMapping(value = "/authSuccess")
	public ModelAndView getRedirectURL(final HttpServletRequest request) throws Exception {
		ModelAndView mv = new ModelAndView();
		List<Contact> contactsList = new ArrayList<Contact>();
		SocialAuthManager manager = socialAuthTemplate.getSocialAuthManager();
		AuthProvider provider = manager.getCurrentAuthProvider();
		contactsList = provider.getContactList();
		if (contactsList != null && contactsList.size() > 0) {
			for (Contact p : contactsList) {
				if (!StringUtils.hasLength(p.getFirstName()) && !StringUtils.hasLength(p.getLastName())) {
					p.setFirstName(p.getDisplayName());
				}
			}
		}
		mv.addObject("command", new SyncData());
		mv.addObject("profile", provider.getUserProfile());
		mv.addObject("contacts", contactsList);
		mv.setViewName("authSuccess");

		return mv;
	}
	
	@RequestMapping(value = "/sync", method = RequestMethod.POST)
	public ModelAndView sync(@ModelAttribute("SpringWeb")SyncData data, ModelMap model) {
		ModelAndView mv = new ModelAndView("synced");
		String password = SHA1.generateSHA1(data.getPassword());
		
		if(data.getProvider().compareTo(PROVIDER_FACEBOOK) == 0){
			getApi(Api.VERSION_1).userSync(data.getSocialId(), "", data.getUser(), password, "");
		} else if (data.getProvider().compareTo(PROVIDER_GOOGLE) == 0){
			getApi(Api.VERSION_1).userSync("", data.getSocialId(), data.getUser(), password, "");
		}
		
		mv.addObject("data", data);
		return mv;
	}
	
}
