package cz.ceskahospodynka.api.mybatis;

import java.util.List;
import java.util.Set;

import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import cz.ceskahospodynka.api.pojo.Article;
import cz.ceskahospodynka.api.pojo.Category;
import cz.ceskahospodynka.api.pojo.Link;
import cz.ceskahospodynka.api.pojo.Menu;
import cz.ceskahospodynka.api.pojo.Opinion;
import cz.ceskahospodynka.api.pojo.Recipe;

public interface Mapper {

	// KATEGORIE
	
	@Select("SELECT * FROM popis_kategorie")
	public List<Category> selectAllCategories();

	//@Select("SELECT kategorie, obrazek FROM popis_kategorie WHERE (category_utf = #{category} OR kategorie = #{category})")
	@Select("SELECT kategorie, obrazek FROM popis_kategorie WHERE (kategorie = #{category})")
	public Category selectCategory(@Param("category") String category);
	
	@Select("SELECT MD5(CONCAT(kategorie, obrazek)) FROM popis_kategorie;")
	public List<String> selectHashes();
	
	// RECEPTY
	
	@Select("SELECT * FROM recept WHERE (kategorie = #{category}) ORDER BY ${order} LIMIT #{start}, #{limit};")
	public Set<Recipe> selectRecipes(
			@Param("category") String category, 
			@Param("order") String order,
			@Param("start") Integer start, 
			@Param("limit") Integer limit);
	
	@Select("SELECT * FROM recept WHERE UPPER(nazev) LIKE  '%${searchString}%' ORDER BY body DESC LIMIT #{start}, #{limit};")
	public Set<Recipe> selectSearchRecipes(
			@Param("searchString") String searchString, 
			@Param("start") Integer start, 
			@Param("limit") Integer limit);
	
	@Select("SELECT COUNT(id) FROM recept WHERE (kategorie = #{category});")
	public Integer selectRecipesCount(@Param("category") String category);
	
	@Select("SELECT COUNT(id) FROM recept WHERE UPPER(nazev) LIKE  '%${searchString}%';")
	public Integer selectSearchRecipesCount(@Param("searchString") String searchString);
	
	@Select("SELECT r.*, COUNT(n.id) AS opinions FROM recept AS r INNER JOIN nazor AS n ON n.id = r.id WHERE r.id = #{id} AND n.status='whitelisted' AND TRIM(n.nazor) != '';")
	public Recipe selectRecipe(@Param("id") Integer id);
	
	@Select("SELECT * FROM recept ORDER BY id DESC LIMIT #{limit};")
	public Set<Recipe> selectNews(@Param("limit") Integer limit);
	
	@Select("SELECT tw.id_recept FROM topweek AS tw WHERE datum BETWEEN adddate(NOW(), -7) AND NOW() GROUP BY tw.id_recept ORDER BY SUM(pocet_navstev) DESC limit #{limit};")
	public Set<Integer> selectTopWatch(@Param("limit") Integer limit);
	
	@Select("SELECT r.* FROM recept AS r ORDER BY body DESC LIMIT #{limit};")
	public Set<Recipe> selectBests(@Param("limit") Integer limit);
	
	@Select("SELECT * FROM recept WHERE  LIMIT #{start}, #{limit};")
	public List<Recipe> findRecipes(
			@Param("category") Integer category, 
			@Param("order") String order,
			@Param("start") Integer start, 
			@Param("limit") Integer limit);
	
	// NAZORY
	
	@Select("SELECT * FROM nazor WHERE id = #{id} AND status='whitelisted' AND TRIM(nazor.nazor) != ''  ORDER BY datum DESC;")
	public List<Opinion> selectOpinions(@Param("id") Integer id);
	
	@Select("SELECT nazor.*, recept.nazev FROM nazor INNER JOIN recept ON recept.id=nazor.id WHERE status='whitelisted' AND TRIM(nazor.nazor) != '' ORDER BY nazor.datum DESC LIMIT 15;")
	public List<Opinion> selectLatestOpinions();
	
	@Select("SELECT nazor.*, recept.nazev FROM nazor INNER JOIN recept ON recept.id=nazor.id WHERE status='whitelisted' AND TRIM(nazor.nazor) != '' ORDER BY nazor.datum DESC LIMIT 1;")
	public Opinion selectLastOpinion();
	
	@Select("INSERT INTO nazor (nazor, podpis, datum, id, status) VALUES (#{opinion}, #{signature}, NOW(), #{recipe}, 'lel');")
	public void saveOpinion(@Param("opinion") String opinion, @Param("signature") String signature, @Param("recipe") Integer recipe);
	
	// MENU
	
	@Select("SELECT * FROM menu ORDER BY body DESC;")
	public List<Menu> selectMenus();
	
	@Select("SELECT * FROM menu ORDER BY body DESC LIMIT 1;")
	public Menu selectLastMenus();
	
	// ODKAZY
	
	@Select("SELECT * FROM odkazy ORDER BY priorita DESC;")
	public List<Link> selectLinks();
	
	@Select("SELECT * FROM odkazy ORDER BY priorita DESC LIMIT 1;")
	public Link selectLastLink();
	
	// CLANKY
	
	@Select("SELECT * FROM fejeton2;")
	public List<Article> selectArticles();
	
	@Select("SELECT * FROM fejeton2 ORDER BY id DESC LIMIT 1;")
	public Article selectLastArticle();
	
	@Select("SELECT * FROM fejeton2 WHERE id = #{id};")
	public Article selectArticle(@Param("id") Integer id);
	
	// ULOZENI OBLIBENENHO RECEPTU
	@Select("INSERT INTO moje (username, receptid) VALUES (#{user}, #{recipeId});")
	public void saveFavorite(@Param("user") String user, @Param("recipeId") Integer recipeId);
	
	// ODEBRANI OBLIBENENHO RECEPTU
	@Select("DELETE FROM moje WHERE username = #{user} AND receptid = #{recipeId};")
	public void dropFavorite(@Param("user") String user, @Param("recipeId") Integer recipeId);
	
	// BODOVANI RECEPTU
	@Select("UPDATE recept SET body = (body+1) WHERE id = #{recipeId};")
	public void updatePoints(@Param("recipeId") Integer recipeId);
	
	// UZIVATEL
	
	@Select("SELECT username FROM social_connector WHERE username = #{username};")
	public String getSocialUser(@Param("username") String userName);
	
	@Select("INSERT INTO social_connector (username, facebook_id, google_id) VALUES (#{user}, #{facebook}, #{google});")
	public void saveSocial(@Param("user") String user, @Param("facebook") String facebook, @Param("google") String google);
	
	@Select("UPDATE social_connector SET google_id = #{google} WHERE username = #{username};")
	public void updateGoogle(@Param("google") String googleId, @Param("username") String userName);
	
	@Select("UPDATE social_connector SET facebook_id = #{facebook} WHERE username = #{username};")
	public void updateFacebook(@Param("facebook") String facebookId, @Param("username") String userName);
	
	@Select("SELECT * FROM users WHERE username = #{username} AND password = #{password};")
	public Integer getUserId(@Param("username") String username, @Param("password") String password);
	
	@Select("SELECT * FROM moje AS m INNER JOIN recept AS r ON r.id = m.receptid WHERE m.username = #{username};")
	public Set<Recipe> getUserRecipes(@Param("username") String username);
}
