package cz.ceskahospodynka.api.utility;

import java.awt.AlphaComposite;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URL;

import javax.imageio.ImageIO;

public class ImageUtils {

	public enum ImageSize{
		SMALL, MEDIUM, ORIGINAL
	};
	
	private static final int IMG_SMALL_WIDTH = 50;
	private static final int IMG_SMALL_HEIGHT = 50;
	
	private static final int IMG_MEDIUM_WIDTH = 150;
	private static final int IMG_MEDIUM_HEIGHT = 150;	

	private static final String web = "http://www.ceskahospodynka.cz/image/x";

	public static String getImageInBase64(String image, boolean usePrefix, ImageSize size) throws IOException {
		byte[] imageBytes = getImage(image, usePrefix, size);
		return Base64.encodeToString(imageBytes, Base64.DEFAULT);
	}
	
	public static byte[] getImage(String imagePath, boolean usePrefix, ImageSize size) throws IOException{
		URL url = new URL((usePrefix ? web : "") + imagePath);
		BufferedImage originalImage = ImageIO.read(url);
		
		if(size != ImageSize.ORIGINAL){
			int type = originalImage.getType() == 0 ? BufferedImage.TYPE_INT_ARGB : originalImage.getType();
			BufferedImage resizeImageJpg = resizeImageWithHint(originalImage, type, size);
			originalImage = resizeImageJpg;
		}
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ImageIO.write(originalImage, "jpg", baos);
		try {
			return baos.toByteArray();
		} finally {
			baos.close();
		}
	}
	
	private static BufferedImage resizeImageWithHint(BufferedImage originalImage, int type, ImageSize size) {

		int width = 0;
		int height = 0;
		
		switch (size) {
		case SMALL:
			width = IMG_SMALL_WIDTH;
			height = IMG_SMALL_HEIGHT;
			break;
		case MEDIUM:
			width = IMG_MEDIUM_WIDTH;
			height = IMG_MEDIUM_HEIGHT;
			break;
		default:
			break;
		}
		
		Dimension newImageSize = getScaledDimension(new Dimension(originalImage.getWidth(), originalImage.getHeight()), new Dimension(width, height));
		
		width = (int)newImageSize.getWidth();
		height = (int)newImageSize.getHeight();
		
		BufferedImage resizedImage = new BufferedImage(width, height, type);
		Graphics2D g = resizedImage.createGraphics();
		g.drawImage(originalImage, 0, 0, width, height, null);
		g.dispose();
		g.setComposite(AlphaComposite.Src);

		g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
		g.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		return resizedImage;
	}
	
	public static Dimension getScaledDimension(Dimension imgSize, Dimension boundary) {

	    int original_width = imgSize.width;
	    int original_height = imgSize.height;
	    int bound_width = boundary.width;
	    int bound_height = boundary.height;
	    int new_width = original_width;
	    int new_height = original_height;

	    // first check if we need to scale width
	    if (original_width > bound_width) {
	        //scale width to fit
	        new_width = bound_width;
	        //scale height to maintain aspect ratio
	        new_height = (new_width * original_height) / original_width;
	    }

	    // then check if we need to scale even with the new height
	    if (new_height > bound_height) {
	        //scale height to fit instead
	        new_height = bound_height;
	        //scale width to maintain aspect ratio
	        new_width = (new_height * original_width) / original_height;
	    }

	    return new Dimension(new_width, new_height);
	}
	
	public static String getImageSize(int size){
		double bytes = size;
		double kilobytes = (bytes / 1024);
		double megabytes = (kilobytes / 1024);

		String ret = "";
		ret += "bytes : " + bytes + " - ";
		ret += "kilobytes : " + kilobytes + " - ";
		ret += "megabytes : " + megabytes + " - ";
		return ret;
	}
}
