package cz.ceskahospodynka.api.utility.cache;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletResponse;

import cz.ceskahospodynka.api.utility.CacheConfigParameter;
import cz.ceskahospodynka.api.utility.Cacheability;
import cz.ceskahospodynka.api.utility.HTTPCacheHeader;

public class CacheFilter implements Filter {
	
	private Cacheability cacheability;

	private boolean isStatic;

	private long seconds;

	public void init(FilterConfig filterConfig) throws ServletException {
		cacheability = (Boolean.valueOf(filterConfig.getInitParameter(CacheConfigParameter.PRIVATE.getName()))) ? Cacheability.PRIVATE : Cacheability.PUBLIC;
		isStatic = Boolean.valueOf(filterConfig.getInitParameter(CacheConfigParameter.STATIC.getName()));

		try {
			seconds = Long.valueOf(filterConfig.getInitParameter(CacheConfigParameter.EXPIRATION_TIME.getName()));
		} catch (NumberFormatException e) {
			throw new ServletException(new StringBuilder("The initialization parameter ")
			.append(CacheConfigParameter.EXPIRATION_TIME.getName())
			.append(" is missing for filter ")
			.append(filterConfig.getFilterName()).append(".")
			.toString());
		}
	}

	public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain) throws IOException, ServletException {
		HttpServletResponse httpServletResponse = (HttpServletResponse) servletResponse;
		StringBuilder cacheControl = new StringBuilder(cacheability.getValue()).append(", max-age=").append(seconds);
		//HttpServletRequest httpservletrequest = (HttpServletRequest) servletRequest;

		if (!isStatic) {
			cacheControl.append(", must-revalidate");
		}

		httpServletResponse.setHeader(HTTPCacheHeader.CACHE_CONTROL.getName(), cacheControl.toString());
		httpServletResponse.setDateHeader(HTTPCacheHeader.EXPIRES.getName(), System.currentTimeMillis() + seconds * 1000L);

		if (httpServletResponse.containsHeader("Pragma")) {
			httpServletResponse.setHeader(HTTPCacheHeader.PRAGMA.getName(), null);
		}

		filterChain.doFilter(servletRequest, servletResponse);
	}

	public void destroy() {
	}
}
