package cz.ceskahospodynka.api.version;

import java.io.IOException;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;

import cz.ceskahospodynka.api.controller.BaseApi;
import cz.ceskahospodynka.api.mybatis.Mapper;
import cz.ceskahospodynka.api.pojo.Article;
import cz.ceskahospodynka.api.pojo.ArticleList;
import cz.ceskahospodynka.api.pojo.Category;
import cz.ceskahospodynka.api.pojo.CategoryList;
import cz.ceskahospodynka.api.pojo.DefaultApi;
import cz.ceskahospodynka.api.pojo.Link;
import cz.ceskahospodynka.api.pojo.LinkList;
import cz.ceskahospodynka.api.pojo.Menu;
import cz.ceskahospodynka.api.pojo.MenuList;
import cz.ceskahospodynka.api.pojo.MyRecipe;
import cz.ceskahospodynka.api.pojo.Opinion;
import cz.ceskahospodynka.api.pojo.OpinionList;
import cz.ceskahospodynka.api.pojo.Recipe;
import cz.ceskahospodynka.api.pojo.RecipeList;
import cz.ceskahospodynka.api.pojo.Title;
import cz.ceskahospodynka.api.pojo.User;
import cz.ceskahospodynka.api.utility.AES;
import cz.ceskahospodynka.api.utility.ImageUtils;
import cz.ceskahospodynka.api.utility.ImageUtils.ImageSize;


public class Version1 extends BaseApi implements Api {

	public Version1(Mapper mapper) {
		super(mapper);
	}
	
	@Override
	public String getTitle() {
		Title title = new Title();
		
		title.setLink(mapper.selectLastLink());
		title.setArticle(mapper.selectLastArticle());
		title.setMenu(mapper.selectLastMenus());
		title.setOpinion(mapper.selectLastOpinion());
		
		return gson.toJson(title);
	}

	@Override
	public String getArticles() {
		final ArticleList articleList = new ArticleList();

		final List<Article> list = mapper.selectArticles();
		for(Article a : list){
			Article article = new Article();
			
			article.setId(a.getId());
			article.setNadpis(a.getNadpis());
			article.setFejeton(a.getFejeton().substring(0, a.getFejeton().length() < 100 ? a.getFejeton().length() : 100));
			
			articleList.getArticleList().add(article);
		}
		
		return gson.toJson(articleList);
	}

	@Override
	public String getArticle(Integer id) {
		Article article = mapper.selectArticle(id);
		return gson.toJson(article);
	}

	@Override
	public String getCategories() {
		final CategoryList categoryList = new CategoryList();

		final List<Category> listWithImages = new LinkedList<Category>();
		
		final List<Category> list = mapper.selectAllCategories();
		for(Category category : list){
			Category withBaseImage = new Category();
			
			withBaseImage.setKategorie(AES.encrypt(category.getKategorie()));
			withBaseImage.setObrazek(AES.encrypt(category.getObrazek()));
			
			try {
				withBaseImage.setObrazekBase64(ImageUtils.getImageInBase64(category.getObrazek(), false, ImageSize.ORIGINAL));
			} catch (IOException e) {}
			
			listWithImages.add(withBaseImage);
		}
		
		categoryList.setCategoryList(listWithImages);

		return gson.toJson(categoryList);
	}
	
	@Override
	public String getCategoriesHash() {
		List<String> hashes = mapper.selectHashes();
		return gson.toJson(hashes);
	}

	@Override
	public String getLinks() {
		final LinkList linkList = new LinkList();
		final List<Link> list = mapper.selectLinks();
		linkList.setLinkList(list);

		return gson.toJson(linkList);
	}

	@Override
	public String getMenus() {
		final MenuList menuList = new MenuList();

		final List<Menu> list = mapper.selectMenus();
		for(Menu m : list){
			Menu menu = m;
			menuList.getMenuList().add(menu);
		}
		menuList.setMenuList(list);
		return gson.toJson(menuList);
	}

	@Override
	public String getLatestOpinions() {
		final OpinionList opinionList = new OpinionList();
		final List<Opinion> list = mapper.selectLatestOpinions();
		opinionList.setOpinionList(list);
		return gson.toJson(opinionList);
	}

	@Override
	public String getRecipeOpinions(Integer recipeId) {
		final OpinionList opinionList = new OpinionList();
		final List<Opinion> list = mapper.selectOpinions(recipeId);
		
		opinionList.setOpinionList(list);
		
		return gson.toJson(opinionList);
	}

	@Override
	public String getRecipes(String category, Integer start, Integer limit, String order) {
		final RecipeList recipeList = new RecipeList();

		final Set<Recipe> list = mapper.selectRecipes(category, order, start, limit);
		final Set<Recipe> cryptedList = new HashSet<Recipe>();
		
		for(Recipe recipe : list){
			cryptedList.add(cryptRecipe(recipe));
		}
		
		recipeList.setCount(mapper.selectRecipesCount(category));
		recipeList.setRecipeList(cryptedList);

		return gson.toJson(recipeList);
	}
	
	@Override
	public String searchRecipes(String searchString, Integer start, Integer limit) {
		final RecipeList recipeList = new RecipeList();

		final Set<Recipe> list = mapper.selectSearchRecipes(searchString.toUpperCase(), start, limit);
		final Set<Recipe> cryptedList = new HashSet<Recipe>();
		
		for(Recipe recipe : list){
			cryptedList.add(cryptRecipe(recipe));
		}
		
		recipeList.setCount(mapper.selectSearchRecipesCount(searchString.toUpperCase()));
		recipeList.setRecipeList(cryptedList);

		return gson.toJson(recipeList);
	}

	@Override
	public String getRecipe(Integer recipeId) {
		final Recipe recipe = mapper.selectRecipe(recipeId);
		Recipe cryptRecipe = cryptRecipe(recipe);
		return gson.toJson(cryptRecipe);
	}

	@Override
	public String getNewRecipes(Integer limit) {
		final RecipeList recipeList = new RecipeList();
		final Set<Recipe> list = mapper.selectNews(limit);
		final Set<Recipe> cryptedList = new HashSet<Recipe>();
		
		for(Recipe recipe : list){
			cryptedList.add(cryptRecipe(recipe));
		}
		recipeList.setRecipeList(cryptedList);
		return gson.toJson(recipeList);
	}

	@Override
	public String getTopwatchRecipe(Integer limit){
		final RecipeList recipeList = new RecipeList();

		final Set<Integer> list = mapper.selectTopWatch(limit);
		final Set<Recipe> cryptedList = new HashSet<Recipe>();
		
		for(Integer recipeId : list){
			Recipe recipe = mapper.selectRecipe(recipeId);
			cryptedList.add(cryptRecipe(recipe));
		}
		recipeList.setRecipeList(cryptedList);
		return gson.toJson(recipeList);
	}

	@Override
	public String getBestRecipes(Integer limit) {
		final RecipeList recipeList = new RecipeList();

		final Set<Recipe> list = mapper.selectBests(limit);
		final Set<Recipe> cryptedList = new HashSet<Recipe>();
		
		for(Recipe recipe : list){
			cryptedList.add(cryptRecipe(recipe));
		}
		
		recipeList.setRecipeList(cryptedList);

		return gson.toJson(recipeList);
	}

	@Override
	public ResponseEntity<byte[]> getRecipeImage(String type, Integer id) throws IOException {
		return new ResponseEntity<byte[]>(ImageUtils.getImage(getRecipeImg(id), true, getSize(type)), getHeader(), HttpStatus.CREATED);
	}
	
	private String getRecipeImg(Integer id){
		final Recipe recipe = mapper.selectRecipe(id);
		return recipe.getObrazek();
	}
	
	private HttpHeaders getHeader(){
		HttpHeaders headers = new HttpHeaders();
	    headers.setContentType(MediaType.IMAGE_JPEG);
	    return headers;
	}
	
	@Override
	public String saveFavoriteRecipe(String user, Integer recipeId) {
		mapper.saveFavorite(user, recipeId);
		DefaultApi api = new DefaultApi();
		return gson.toJson(api);
	}
	
	@Override
	public String dropFavoriteRecipe(String user, Integer recipeId) {
		mapper.dropFavorite(user, recipeId);
		DefaultApi api = new DefaultApi();
		return gson.toJson(api);
	}

	@Override
	public String savePoint(Integer recipeId) {
		mapper.updatePoints(recipeId);
		
		DefaultApi api = new DefaultApi();
		return gson.toJson(api);
	}
	
	@Override
	public String login(String user, String password) {
		User dbUser = new User();
		dbUser.setUserId(mapper.getUserId(user, password));
		return gson.toJson(dbUser);
	}
	
	@Override
	public String userSync(String facebookId, String googleId, String user, String password, String recipes) {
		
		if(user.length() > 0 && (facebookId.length() > 0 || googleId.length() > 0) && login(user, password) != null){
			// synchronizuji social_connector
			if(mapper.getSocialUser(user) != null){
				// upravim
				if(facebookId.length() > 0){
					mapper.updateFacebook(facebookId, user);
				}
				if(googleId.length() > 0){
					mapper.updateGoogle(googleId, user);
				}
			} else {
				// vlozim novy radek
				mapper.saveSocial(user, facebookId, googleId);
			}
		}
		
		String userId = null;
		if(user.length() > 0){
			userId = user;
		} else if(facebookId.length() > 0){
			userId = facebookId;
		} else if(googleId.length() > 0){
			userId = googleId;
		}
		
		if(userId != null && recipes.length() > 0){
			MyRecipe saveRecipes = gson.fromJson(recipes, MyRecipe.class);
			
			for(Integer recipe : saveRecipes.getRecipesId()){
				try{
					mapper.saveFavorite(userId, recipe);
				} catch (Exception e){}
			}
		}
		
		final RecipeList recipeList = new RecipeList();

		final Set<Recipe> list = mapper.getUserRecipes(user);
		list.addAll(mapper.getUserRecipes(facebookId));
		list.addAll(mapper.getUserRecipes(googleId));
		
		final Set<Recipe> cryptedList = new HashSet<Recipe>();
			
		for(Recipe recipe : list){
			cryptedList.add(cryptRecipe(recipe));
		}
			
		recipeList.setRecipeList(cryptedList);

		return gson.toJson(recipeList);
	}
	
	@Override
	public String saveOpinion(Integer recipe, String signature, String opinion) {
		mapper.saveOpinion(opinion, signature, recipe);
		DefaultApi api = new DefaultApi();
		return gson.toJson(api);
	}
}
